/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;

public class BuildPath extends Composite implements SelectionListener {

	/**
	 * @param parent
	 * @param style
	 */
	public BuildPath(Composite parent, int style) {
		super(parent, style);
		initialise();
	}

	public void commit() {
		if (projectRadio.getSelection()) {
			PreferenceManager.setProjectFolderOutput(true);
		} else if (foldersRadio.getSelection()) {
			PreferenceManager.setProjectFolderOutput(false);
		}
		PreferenceManager.setOutputFolders(new String[] { sourceText.getText(),
				objectText.getText(), docText.getText(), htmlText.getText() });
	}

	public void revertToDefaults() {
		setWidgetValues(PreferenceManager.defaultProjectFolderOutput(),
				PreferenceManager.getDefaultOutputFolders());
	}

	private void initialise() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		GridLayout layout = new GridLayout();
		this.setLayout(layout);
		this.setLayoutData(gridData);
		createFolderGroup();
		setWidgetValues(PreferenceManager.projectFolderOutput(),
				PreferenceManager.getOutputFolders());
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createFolderGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		folderGroup = new Group(this, SWT.NONE);
		folderGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SourceOutputFolders")); //$NON-NLS-1$
		folderGroup.setLayout(layout);
		folderGroup.setLayoutData(gridData);
		createRadioButtons();
		createSourceEntry();
		createObjectEntry();
		createDocEntry();
		createHtmlEntry();
	}

	private void createRadioButtons() {
		projectRadio = new Button(folderGroup, SWT.RADIO);
		projectRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Project")); //$NON-NLS-1$
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		projectRadio.setLayoutData(gridData);
		projectRadio.addSelectionListener(this);
		foldersRadio = new Button(folderGroup, SWT.RADIO);
		foldersRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Folders")); //$NON-NLS-1$
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		foldersRadio.setLayoutData(gridData);
		foldersRadio.addSelectionListener(this);
	}

	private void createSourceEntry() {
		sourceLabel = new Label(folderGroup, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalIndent = 22;
		sourceLabel.setLayoutData(gridData);
		sourceLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SourceFolderName")
				+ ":");
		sourceText = new Text(folderGroup, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 1;
		sourceText.setLayoutData(gridData);
	}

	private void createObjectEntry() {
		objectLabel = new Label(folderGroup, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalIndent = 22;
		objectLabel.setLayoutData(gridData);
		objectLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ObjectFolderName")
				+ ":");
		objectText = new Text(folderGroup, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 1;
		objectText.setLayoutData(gridData);
	}

	private void createDocEntry() {
		docLabel = new Label(folderGroup, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalIndent = 22;
		docLabel.setLayoutData(gridData);
		docLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.DocFolderName")
				+ ":");
		docText = new Text(folderGroup, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 1;
		docText.setLayoutData(gridData);
	}

	private void createHtmlEntry() {
		htmlLabel = new Label(folderGroup, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalIndent = 22;
		htmlLabel.setLayoutData(gridData);
		htmlLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.HtmlFolderName")
				+ ":");
		htmlText = new Text(folderGroup, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 1;
		htmlText.setLayoutData(gridData);
	}

	private void setWidgetValues(boolean projectFolder, String[] outputFolders) {
		sourceText.setText(outputFolders[0]);
		objectText.setText(outputFolders[1]);
		docText.setText(outputFolders[2]);
		htmlText.setText(outputFolders[3]);
		if (projectFolder) {
			projectRadio.setSelection(true);
			foldersRadio.setSelection(false);
			sourceText.setEnabled(false);
			objectText.setEnabled(false);
			docText.setEnabled(false);
			htmlText.setEnabled(false);
			sourceLabel.setEnabled(false);
			objectLabel.setEnabled(false);
			docLabel.setEnabled(false);
			htmlLabel.setEnabled(false);
		} else {
			projectRadio.setSelection(false);
			foldersRadio.setSelection(true);
			sourceText.setEnabled(true);
			objectText.setEnabled(true);
			docText.setEnabled(true);
			htmlText.setEnabled(true);
			sourceLabel.setEnabled(true);
			objectLabel.setEnabled(true);
			docLabel.setEnabled(true);
			htmlLabel.setEnabled(true);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == projectRadio) {
			projectRadio.setSelection(true);
			foldersRadio.setSelection(false);
			sourceText.setEnabled(false);
			objectText.setEnabled(false);
			docText.setEnabled(false);
			htmlText.setEnabled(false);
			sourceLabel.setEnabled(false);
			objectLabel.setEnabled(false);
			docLabel.setEnabled(false);
			htmlLabel.setEnabled(false);
		} else if (event.getSource() == foldersRadio) {
			projectRadio.setSelection(false);
			foldersRadio.setSelection(true);
			sourceText.setEnabled(true);
			objectText.setEnabled(true);
			docText.setEnabled(true);
			htmlText.setEnabled(true);
			sourceLabel.setEnabled(true);
			objectLabel.setEnabled(true);
			docLabel.setEnabled(true);
			htmlLabel.setEnabled(true);
		}
	}

	private Group folderGroup = null;

	private Button projectRadio = null;

	private Button foldersRadio = null;

	private Label sourceLabel = null;

	private Text sourceText = null;

	private Label objectLabel = null;

	private Text objectText = null;

	private Label docLabel = null;

	private Text docText = null;

	private Label htmlLabel = null;

	private Text htmlText = null;
}
