/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;

public class ECLiPSeChangeDepthShell {

	public final static int PRINT = 0;

	public final static int DEBUG = 0;

	public final static int STRUCTURE = 0;

	private Shell changeDepthShell = null;

	private String title = null;

	private Composite changeDepthComposite = null;

	/**
	 * This method initializes changeDepthShell
	 * 
	 * The shell is a modal window, so that the parent window is blocked until
	 * the print depth is selected or the selection cancelled The input
	 * parameter title contains the string that is displayed as the window title
	 */
	private void createChangeDepthShell(Shell parent, String title, int value,
			boolean fullDepth) {
		changeDepthShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		changeDepthShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSe") + title); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		changeDepthShell.setLayout(layout);
		changeDepthComposite = new ECLiPSeChangeDepth(changeDepthShell,
				SWT.NONE, title, value, fullDepth, true);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		changeDepthComposite.setLayoutData(gridData);
		changeDepthShell.setSize(changeDepthShell.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
	}

	public int open(Shell parent, int mode) {
		int value = open(parent, mode, 20, false);
		return value;
	}

	public int open(Shell parent, int mode, int value, boolean fullDepth) {
		// get window title
		if (mode == PRINT) {
			title = PreferenceMessages.getString("ECLiPSeStrings.PrintDepth");
		} else if (mode == DEBUG) {
			title = PreferenceMessages
					.getString("ECLiPSeStrings.DebugPrintDepth");
		} else if (mode == STRUCTURE) {
			title = PreferenceMessages
					.getString("ECLiPSeStrings.StructureThreshold");
		}
		// open shell
		createChangeDepthShell(parent, title, value, fullDepth);
		Point parentSize = parent.getSize();
		Point size = changeDepthShell.getSize();
		int x = (parentSize.x - size.x) / 2;
		int y = (parentSize.y - size.y) / 2;
		changeDepthShell.setLocation(parent.toDisplay(x, y));
		changeDepthShell.open();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!changeDepthShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return ((ECLiPSeChangeDepth) changeDepthComposite).getPrintDepth();
	}

	/**
	 * main method, for debugging purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		ECLiPSeChangeDepthShell thisClass = new ECLiPSeChangeDepthShell();
		thisClass.open(null, PRINT);

		while (!thisClass.changeDepthShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}
}
