/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class ECLiPSeEngineSettings extends Composite implements
		SelectionListener {

	// private EnginePreferences installation = null;

	/**
	 * @param parent
	 * @param style
	 * @param installation
	 */
	public ECLiPSeEngineSettings(Composite parent, int style,
			boolean printOutput, EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		initialise(installation, printOutput, false);
	}

	/**
	 * @param parent
	 * @param style
	 * @param installation
	 */
	public ECLiPSeEngineSettings(Composite parent, int style,
			boolean printOutput, boolean shellButtons,
			EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		initialise(installation, printOutput, shellButtons);
	}

	public EnginePreferences commit(EnginePreferences installation) {
		if (timerRealRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.afterEventTimer,
					PreferenceConstants.real);

		} else if (timerVirtualRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.afterEventTimer,
					PreferenceConstants.virtual);
		}

		if (allDynamicOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.allDynamic,
					PreferenceConstants.on);
		} else if (allDynamicOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.allDynamic,
					PreferenceConstants.off);
		}

		if (brealExceptOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.brealExceptions,
					PreferenceConstants.on);
		} else if (brealExceptOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.brealExceptions,
					PreferenceConstants.off);
		}

		if (coroutineOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.coroutine,
					PreferenceConstants.on);
		} else if (coroutineOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.coroutine,
					PreferenceConstants.off);
		}

		if (debuggingCreepRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.debugging,
					PreferenceConstants.creep);
		} else if (debuggingNoDebugRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.debugging,
					PreferenceConstants.nodebug);
		} else if (debuggingLeapRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.debugging,
					PreferenceConstants.leap);
		}

		if (debugCompileOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.debugCompile,
					PreferenceConstants.on);
		} else if (debugCompileOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.debugCompile,
					PreferenceConstants.off);
		}

		if (interruptsOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.enableInterrupts,
					PreferenceConstants.on);
		} else if (interruptsOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.enableInterrupts,
					PreferenceConstants.off);
		}

		if (goalExpansionOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.goalExpansion,
					PreferenceConstants.on);
		} else if (goalExpansionOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.goalExpansion,
					PreferenceConstants.off);
		}

		if (macroExpansionOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.macroExpansion,
					PreferenceConstants.on);
		} else if (macroExpansionOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.macroExpansion,
					PreferenceConstants.off);
		}

		if (rationalsOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.preferRationals,
					PreferenceConstants.on);
		} else if (rationalsOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.preferRationals,
					PreferenceConstants.off);
		}

		if (varNamesOnRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.variableNames,
					PreferenceConstants.on);
		} else if (varNamesOffRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.variableNames,
					PreferenceConstants.off);
		} else if (varNamesSingletonsRadio.getSelection()) {
			installation.setFlag(PreferenceConstants.variableNames,
					PreferenceConstants.checkSingletons);
		}

		if (printOutput) {
			installation = printOutputComposite.commit(installation);
		}
		return installation;
	}

	public EnginePreferences revertToDefaults(EnginePreferences installation) {
		String[] flagNames = new String[] {
				PreferenceConstants.afterEventTimer,
				PreferenceConstants.allDynamic,
				PreferenceConstants.brealExceptions,
				PreferenceConstants.coroutine, PreferenceConstants.debugging,
				PreferenceConstants.debugCompile,
				PreferenceConstants.enableInterrupts,
				PreferenceConstants.goalExpansion,
				PreferenceConstants.macroExpansion,
				PreferenceConstants.preferRationals,
				PreferenceConstants.variableNames };
		PreferenceManager.revertToDefault(installation, flagNames);
		setWidgetValues(installation);
		if (printOutput) {
			installation = printOutputComposite.revertToDefaults(installation);
		}
		return installation;
	}

	private void initialise(EnginePreferences installation,
			boolean printOutput, boolean shellButtons) {
		this.printOutput = printOutput;
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		this.setLayout(layout);
		this.setLayoutData(gridData);
		createEngineSettingsGroup(installation);
		setWidgetValues(installation);
		if (shellButtons) {
			createButtonComposite();
		}
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createButtonComposite() {
		buttonComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = true;
		buttonComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		buttonComposite.setLayoutData(gridData);
		okButton = new Button(buttonComposite, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
		cancelButton = new Button(buttonComposite, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
		cancelButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		cancelButton.setLayoutData(gridData);
		okButton.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		okButton.setLayoutData(gridData);
	}

	private void createEngineSettingsGroup(EnginePreferences installation) {
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		engineSettingsGroup = new Group(this, SWT.NONE);
		engineSettingsGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngineSettings")); //$NON-NLS-1$
		engineSettingsGroup.setLayout(layout);
		engineSettingsGroup.setLayoutData(gridData);
		createEngineSettingsComposite(installation);
	}

	private void createEngineSettingsComposite(EnginePreferences installation) {
		GridLayout layout = new GridLayout();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		engineSettingsComposite = new Composite(engineSettingsGroup, SWT.NONE);
		engineSettingsComposite.setLayout(layout);
		engineSettingsComposite.setLayoutData(gridData);
		createTimerEntry();
		createAllDynamicEntry();
		createBrealExceptEntry();
		createCoroutineEntry();
		createDebuggingEntry();
		createDebugCompileEntry();
		createInterruptsEntry();
		createGoalExpansionEntry();
		createMacroExpansionEntry();
		createRationalsEntry();
		createVarNamesEntry();
		if (printOutput) {
			printOutputComposite = new PrintDepthOutputOptionsComposite(
					engineSettingsComposite, SWT.NONE, installation);
		}
		engineSettingsComposite.setSize(engineSettingsComposite.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method creates the timer entry
	 * 
	 */
	private void createTimerEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		timerLabel = new Label(radioGroupComposite, SWT.NONE);
		timerLabel.setText(PreferenceConstants.afterEventTimer + ":"); //$NON-NLS-1$
		timerRealRadio = new Button(radioGroupComposite, SWT.RADIO);
		timerRealRadio.setText(PreferenceConstants.real); //$NON-NLS-1$
		timerVirtualRadio = new Button(radioGroupComposite, SWT.RADIO);
		timerVirtualRadio.setText(PreferenceConstants.virtual); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, timerLabel,
				timerRealRadio, timerVirtualRadio, null);
	}

	/**
	 * This method creates the allDynamic entry
	 * 
	 */
	private void createAllDynamicEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		allDynamicLabel = new Label(radioGroupComposite, SWT.NONE);
		allDynamicLabel.setText(PreferenceConstants.allDynamic + ":"); //$NON-NLS-1$
		allDynamicOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		allDynamicOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		allDynamicOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		allDynamicOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, allDynamicLabel,
				allDynamicOffRadio, allDynamicOnRadio, null);
	}

	/**
	 * This method creates the brealExcept entry
	 * 
	 */
	private void createBrealExceptEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		brealExceptLabel = new Label(radioGroupComposite, SWT.NONE);
		brealExceptLabel.setText(PreferenceConstants.brealExceptions + ":"); //$NON-NLS-1$
		brealExceptOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		brealExceptOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		brealExceptOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		brealExceptOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, brealExceptLabel,
				brealExceptOffRadio, brealExceptOnRadio, null);
	}

	/**
	 * This method creates the coroutine entry
	 * 
	 */
	private void createCoroutineEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		coroutineLabel = new Label(radioGroupComposite, SWT.NONE);
		coroutineLabel.setText(PreferenceConstants.coroutine + ":"); //$NON-NLS-1$
		coroutineOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		coroutineOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		coroutineOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		coroutineOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, coroutineLabel,
				coroutineOffRadio, coroutineOnRadio, null);
	}

	/**
	 * This method creates the debugging entry
	 * 
	 */
	private void createDebuggingEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		debuggingLabel = new Label(radioGroupComposite, SWT.NONE);
		debuggingLabel.setText(PreferenceConstants.debugging + ":"); //$NON-NLS-1$
		debuggingNoDebugRadio = new Button(radioGroupComposite, SWT.RADIO);
		debuggingNoDebugRadio.setText(PreferenceConstants.nodebug); //$NON-NLS-1$
		debuggingCreepRadio = new Button(radioGroupComposite, SWT.RADIO);
		debuggingCreepRadio.setText(PreferenceConstants.creep); //$NON-NLS-1$
		debuggingLeapRadio = new Button(radioGroupComposite, SWT.RADIO);
		debuggingLeapRadio.setText(PreferenceConstants.leap); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, debuggingLabel,
				debuggingNoDebugRadio, debuggingCreepRadio, debuggingLeapRadio);
	}

	/**
	 * This method creates the debugCompile entry
	 * 
	 */
	private void createDebugCompileEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		debugCompileLabel = new Label(radioGroupComposite, SWT.NONE);
		debugCompileLabel.setText(PreferenceConstants.debugCompile + ":"); //$NON-NLS-1$
		debugCompileOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		debugCompileOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		debugCompileOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		debugCompileOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, debugCompileLabel,
				debugCompileOffRadio, debugCompileOnRadio, null);
	}

	/**
	 * This method creates the interrupts entry
	 * 
	 */
	private void createInterruptsEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		interruptsLabel = new Label(radioGroupComposite, SWT.NONE);
		interruptsLabel.setText(PreferenceConstants.enableInterrupts + ":"); //$NON-NLS-1$
		interruptsOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		interruptsOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		interruptsOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		interruptsOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, interruptsLabel,
				interruptsOffRadio, interruptsOnRadio, null);
	}

	/**
	 * This method creates the goalExpansion entry
	 * 
	 */
	private void createGoalExpansionEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		goalExpansionLabel = new Label(radioGroupComposite, SWT.NONE);
		goalExpansionLabel.setText(PreferenceConstants.goalExpansion + ":"); //$NON-NLS-1$
		goalExpansionOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		goalExpansionOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		goalExpansionOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		goalExpansionOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, goalExpansionLabel,
				goalExpansionOffRadio, goalExpansionOnRadio, null);
	}

	/**
	 * This method creates the macroExpansion entry
	 * 
	 */
	private void createMacroExpansionEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		macroExpansionLabel = new Label(radioGroupComposite, SWT.NONE);
		macroExpansionLabel.setText(PreferenceConstants.macroExpansion + ":"); //$NON-NLS-1$
		macroExpansionOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		macroExpansionOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		macroExpansionOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		macroExpansionOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, macroExpansionLabel,
				macroExpansionOffRadio, macroExpansionOnRadio, null);
	}

	/**
	 * This method creates the rationals entry
	 * 
	 */
	private void createRationalsEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		rationalsLabel = new Label(radioGroupComposite, SWT.NONE);
		rationalsLabel.setText(PreferenceConstants.preferRationals + ":"); //$NON-NLS-1$
		rationalsOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		rationalsOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		rationalsOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		rationalsOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, rationalsLabel,
				rationalsOffRadio, rationalsOnRadio, null);
	}

	/**
	 * This method creates the varNames entry
	 * 
	 */
	private void createVarNamesEntry() {
		Composite radioGroupComposite = new Composite(engineSettingsComposite,
				SWT.NONE);
		varNamesLabel = new Label(radioGroupComposite, SWT.NONE);
		varNamesLabel.setText(PreferenceConstants.variableNames + ":"); //$NON-NLS-1$
		varNamesOffRadio = new Button(radioGroupComposite, SWT.RADIO);
		varNamesOffRadio.setText(PreferenceConstants.off); //$NON-NLS-1$
		varNamesOnRadio = new Button(radioGroupComposite, SWT.RADIO);
		varNamesOnRadio.setText(PreferenceConstants.on); //$NON-NLS-1$
		varNamesSingletonsRadio = new Button(radioGroupComposite, SWT.RADIO);
		varNamesSingletonsRadio.setText(PreferenceConstants.checkSingletons); //$NON-NLS-1$
		RadioCompositeBuilder.build(radioGroupComposite, varNamesLabel,
				varNamesOffRadio, varNamesOnRadio, varNamesSingletonsRadio);
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			parentComposite.dispose();
		} else if (event.getSource() == cancelButton) {
			parentComposite.dispose();
		}
	}

	public void setWidgetValues(EnginePreferences installation) {
		String timerSelection = installation
				.getStringFlag(PreferenceConstants.afterEventTimer);
		if (timerSelection.equals(PreferenceConstants.real)) {
			timerRealRadio.setSelection(true);
			timerVirtualRadio.setSelection(false);
		} else if (timerSelection.equals(PreferenceConstants.virtual)) {
			timerRealRadio.setSelection(false);
			timerVirtualRadio.setSelection(true);
		} else {
			timerRealRadio.setSelection(false);
			timerVirtualRadio.setSelection(false);
		}
		String allDynamic = installation
				.getStringFlag(PreferenceConstants.allDynamic);
		if (allDynamic.equals(PreferenceConstants.on)) {
			allDynamicOnRadio.setSelection(true);
			allDynamicOffRadio.setSelection(false);
		} else if (allDynamic.equals(PreferenceConstants.off)) {
			allDynamicOnRadio.setSelection(false);
			allDynamicOffRadio.setSelection(true);
		} else {
			allDynamicOnRadio.setSelection(false);
			allDynamicOffRadio.setSelection(false);
		}
		String brealExcept = installation
				.getStringFlag(PreferenceConstants.brealExceptions);
		if (brealExcept.equals(PreferenceConstants.on)) {
			brealExceptOnRadio.setSelection(true);
			brealExceptOffRadio.setSelection(false);
		} else if (brealExcept.equals(PreferenceConstants.off)) {
			brealExceptOnRadio.setSelection(false);
			brealExceptOffRadio.setSelection(true);
		} else {
			brealExceptOnRadio.setSelection(false);
			brealExceptOffRadio.setSelection(false);
		}
		String coroutine = installation
				.getStringFlag(PreferenceConstants.coroutine);
		if (coroutine.equals(PreferenceConstants.on)) {
			coroutineOnRadio.setSelection(true);
			coroutineOffRadio.setSelection(false);
		} else if (coroutine.equals(PreferenceConstants.off)) {
			coroutineOnRadio.setSelection(false);
			coroutineOffRadio.setSelection(true);
		} else {
			coroutineOnRadio.setSelection(false);
			coroutineOffRadio.setSelection(false);
		}
		String debugging = installation
				.getStringFlag(PreferenceConstants.debugging);
		if (debugging.equals(PreferenceConstants.creep)) {
			debuggingCreepRadio.setSelection(true);
			debuggingNoDebugRadio.setSelection(false);
			debuggingLeapRadio.setSelection(false);
		} else if (debugging.equals(PreferenceConstants.nodebug)) {
			debuggingCreepRadio.setSelection(false);
			debuggingNoDebugRadio.setSelection(true);
			debuggingLeapRadio.setSelection(false);
		} else if (debugging.equals(PreferenceConstants.leap)) {
			debuggingCreepRadio.setSelection(false);
			debuggingNoDebugRadio.setSelection(false);
			debuggingLeapRadio.setSelection(true);
		} else {
			debuggingCreepRadio.setSelection(false);
			debuggingNoDebugRadio.setSelection(false);
			debuggingLeapRadio.setSelection(false);
		}
		String debugCompile = installation
				.getStringFlag(PreferenceConstants.debugCompile);
		if (debugCompile.equals(PreferenceConstants.on)) {
			debugCompileOnRadio.setSelection(true);
			debugCompileOffRadio.setSelection(false);
		} else if (debugCompile.equals(PreferenceConstants.off)) {
			debugCompileOnRadio.setSelection(false);
			debugCompileOffRadio.setSelection(true);
		} else {
			debugCompileOnRadio.setSelection(false);
			debugCompileOffRadio.setSelection(false);
		}
		String enableInterrupts = installation
				.getStringFlag(PreferenceConstants.enableInterrupts);
		if (enableInterrupts.equals(PreferenceConstants.on)) {
			interruptsOnRadio.setSelection(true);
			interruptsOffRadio.setSelection(false);
		} else if (enableInterrupts.equals(PreferenceConstants.off)) {
			interruptsOnRadio.setSelection(false);
			interruptsOffRadio.setSelection(true);
		} else {
			interruptsOnRadio.setSelection(false);
			interruptsOffRadio.setSelection(false);
		}
		String goalExpansion = installation
				.getStringFlag(PreferenceConstants.goalExpansion);
		if (goalExpansion.equals(PreferenceConstants.on)) {
			goalExpansionOnRadio.setSelection(true);
			goalExpansionOffRadio.setSelection(false);
		} else if (goalExpansion.equals(PreferenceConstants.off)) {
			goalExpansionOnRadio.setSelection(false);
			goalExpansionOffRadio.setSelection(true);
		} else {
			goalExpansionOnRadio.setSelection(false);
			goalExpansionOffRadio.setSelection(false);
		}
		String macroExpansion = installation
				.getStringFlag(PreferenceConstants.macroExpansion);
		if (macroExpansion.equals(PreferenceConstants.on)) {
			macroExpansionOnRadio.setSelection(true);
			macroExpansionOffRadio.setSelection(false);
		} else if (macroExpansion.equals(PreferenceConstants.off)) {
			macroExpansionOnRadio.setSelection(false);
			macroExpansionOffRadio.setSelection(true);
		} else {
			macroExpansionOnRadio.setSelection(false);
			macroExpansionOffRadio.setSelection(false);
		}
		String preferRationals = installation
				.getStringFlag(PreferenceConstants.preferRationals);
		if (preferRationals.equals(PreferenceConstants.on)) {
			rationalsOnRadio.setSelection(true);
			rationalsOffRadio.setSelection(false);
		} else if (preferRationals.equals(PreferenceConstants.off)) {
			rationalsOnRadio.setSelection(false);
			rationalsOffRadio.setSelection(true);
		} else {
			rationalsOnRadio.setSelection(false);
			rationalsOffRadio.setSelection(false);
		}
		String variableNames = installation
				.getStringFlag(PreferenceConstants.variableNames);
		if (variableNames.equals(PreferenceConstants.on)) {
			varNamesOnRadio.setSelection(true);
			varNamesOffRadio.setSelection(false);
			varNamesSingletonsRadio.setSelection(false);
		} else if (variableNames.equals(PreferenceConstants.off)) {
			varNamesOnRadio.setSelection(false);
			varNamesOffRadio.setSelection(true);
			varNamesSingletonsRadio.setSelection(false);
		} else if (variableNames.equals(PreferenceConstants.checkSingletons)) {
			varNamesOnRadio.setSelection(false);
			varNamesOffRadio.setSelection(false);
			varNamesSingletonsRadio.setSelection(true);
		} else {
			varNamesOnRadio.setSelection(false);
			varNamesOffRadio.setSelection(false);
			varNamesSingletonsRadio.setSelection(false);
		}
	}

	public void setEnabled(boolean enabled) {
		engineSettingsGroup.setEnabled(enabled);
		timerLabel.setEnabled(enabled);
		timerRealRadio.setEnabled(enabled);
		timerVirtualRadio.setEnabled(enabled);
		allDynamicLabel.setEnabled(enabled);
		allDynamicOffRadio.setEnabled(enabled);
		allDynamicOnRadio.setEnabled(enabled);
		brealExceptLabel.setEnabled(enabled);
		brealExceptOffRadio.setEnabled(enabled);
		brealExceptOnRadio.setEnabled(enabled);
		coroutineLabel.setEnabled(enabled);
		coroutineOffRadio.setEnabled(enabled);
		coroutineOnRadio.setEnabled(enabled);
		debuggingLabel.setEnabled(enabled);
		debuggingNoDebugRadio.setEnabled(enabled);
		debuggingCreepRadio.setEnabled(enabled);
		debuggingLeapRadio.setEnabled(enabled);
		debugCompileLabel.setEnabled(enabled);
		debugCompileOffRadio.setEnabled(enabled);
		debugCompileOnRadio.setEnabled(enabled);
		interruptsLabel.setEnabled(enabled);
		interruptsOffRadio.setEnabled(enabled);
		interruptsOnRadio.setEnabled(enabled);
		goalExpansionLabel.setEnabled(enabled);
		goalExpansionOffRadio.setEnabled(enabled);
		goalExpansionOnRadio.setEnabled(enabled);
		macroExpansionLabel.setEnabled(enabled);
		macroExpansionOffRadio.setEnabled(enabled);
		macroExpansionOnRadio.setEnabled(enabled);
		rationalsLabel.setEnabled(enabled);
		rationalsOffRadio.setEnabled(enabled);
		rationalsOnRadio.setEnabled(enabled);
		varNamesLabel.setEnabled(enabled);
		varNamesOffRadio.setEnabled(enabled);
		varNamesOnRadio.setEnabled(enabled);
		varNamesSingletonsRadio.setEnabled(enabled);
		if (printOutput) {
			printOutputComposite.setEnabled(enabled);
		}
	}

	private Group engineSettingsGroup = null;

	private Composite engineSettingsComposite = null;

	private Label timerLabel = null;

	private Button timerRealRadio = null;

	private Button timerVirtualRadio = null;

	private Label allDynamicLabel = null;

	private Button allDynamicOffRadio = null;

	private Button allDynamicOnRadio = null;

	private Label brealExceptLabel = null;

	private Button brealExceptOffRadio = null;

	private Button brealExceptOnRadio = null;

	private Label coroutineLabel = null;

	private Button coroutineOffRadio = null;

	private Button coroutineOnRadio = null;

	private Label debuggingLabel = null;

	private Button debuggingNoDebugRadio = null;

	private Button debuggingCreepRadio = null;

	private Button debuggingLeapRadio = null;

	private Label debugCompileLabel = null;

	private Button debugCompileOffRadio = null;

	private Button debugCompileOnRadio = null;

	private Label interruptsLabel = null;

	private Button interruptsOffRadio = null;

	private Button interruptsOnRadio = null;

	private Label goalExpansionLabel = null;

	private Button goalExpansionOffRadio = null;

	private Button goalExpansionOnRadio = null;

	private Label macroExpansionLabel = null;

	private Button macroExpansionOffRadio = null;

	private Button macroExpansionOnRadio = null;

	private Label rationalsLabel = null;

	private Button rationalsOffRadio = null;

	private Button rationalsOnRadio = null;

	private Label varNamesLabel = null;

	private Button varNamesOffRadio = null;

	private Button varNamesOnRadio = null;

	private Button varNamesSingletonsRadio = null;

	private PrintDepthOutputOptionsComposite printOutputComposite = null;

	private boolean printOutput = false;

	private Button cancelButton = null;

	private Button okButton = null;

	private Composite buttonComposite = null;

	private Composite parentComposite = null;
}
