/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;

public class ECLiPSeEngineSettingsShell {

	private Shell engineSettingsShell = null;

	private Composite engineSettingsComposite = null;
	
	private EnginePreferences installation = null;
	
	public ECLiPSeEngineSettingsShell(EnginePreferences installation) {
		this.installation = installation;
	}

	/**
	 * This method initializes engineSettingsShell
	 * 
	 * The shell is a modal window, so that the parent window is blocked until
	 * the selection is made or cancelled The input parameter title contains the
	 * string that is displayed as the window title
	 */
	private void createEngineSettingsShell(Shell parent) {
		engineSettingsShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		engineSettingsShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngineSettings")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		engineSettingsShell.setLayout(layout);
		engineSettingsComposite = new ECLiPSeEngineSettings(
				engineSettingsShell, SWT.NONE, true, installation);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		engineSettingsComposite.setLayoutData(gridData);
		engineSettingsShell.setSize(engineSettingsShell.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method opens a modal ECLiPSeEngineSettingsShell
	 * 
	 * @param parent
	 *            the parent window
	 */
	public void open(Shell parent) {
		// open shell
		createEngineSettingsShell(parent);
		engineSettingsShell.open();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!engineSettingsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * main method, for debugging purposes
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		ECLiPSeEngineSettingsShell thisClass = new ECLiPSeEngineSettingsShell(null);
		thisClass.open(null);
		while (!thisClass.engineSettingsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}
}
