/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Composite;
import java.util.Vector;

public class ECLiPSeOutputOptions extends Composite implements
		SelectionListener {

	/**
	 * default constructor
	 * 
	 * @param parent
	 * @param style
	 * @param installation
	 */
	public ECLiPSeOutputOptions(Composite parent, int style,
			boolean printOutput, EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		initialise(installation, printOutput, false);
	}

	/**
	 * @param parent
	 * @param style
	 * @param flag
	 * @param newCaller
	 * @param installation
	 */
	public ECLiPSeOutputOptions(Composite parent, int style,
			boolean printOutput, boolean showButtons,
			EnginePreferences installation) {
		super(parent, style);
		parentComposite = parent;
		initialise(installation, printOutput, showButtons);
	}

	public EnginePreferences commit(EnginePreferences installation) {
		setOutputModeString(false);
		installation.setFlag(PreferenceConstants.outputMode, outputModeString);
		if (printOutput) {
			installation = printOutputComposite.commit(installation);
			outputModeString = oldOutputModeString = installation
					.getStringFlag(PreferenceConstants.outputMode);
		}
		return installation;
	}

	public EnginePreferences revertToDefaults(EnginePreferences installation) {
		installation = printOutputComposite.revertToDefaults(installation);
		outputModeString = oldOutputModeString = installation
				.getStringFlag(PreferenceConstants.outputMode);
		setWidgetValues(oldOutputModeString);
		return installation;
	}

	/**
	 * this method initializes the ouput options widget
	 * 
	 * @param installation
	 * @param flag
	 */
	private void initialise(EnginePreferences installation,
			boolean printOutput, boolean flag) {
		this.printOutput = printOutput;
		outputModeString = oldOutputModeString = installation
				.getStringFlag(PreferenceConstants.outputMode);
		radioButtons = new Vector();
		createSelectionGroup(installation);
		createButtonComposite(flag);
		setWidgetValues(oldOutputModeString);
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	/**
	 * This method initializes selectionGroup
	 * 
	 */
	private void createSelectionGroup(EnginePreferences installation) {
		this.setLayout(new GridLayout());
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		this.setLayoutData(gridData);
		selectionGroup = new Group(this, SWT.NONE);
		selectionGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.OutputOptions")); //$NON-NLS-1$
		selectionGroup.setLayout(new GridLayout());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		selectionGroup.setLayoutData(gridData);
		createVariablesEntry();
		createAttributesEntry();
		createOperatorsEntry();
		createSpacesEntry();
		createQuotingEntry();
		createListsEntry();
		createPortrayEntry();
		createTransformationsEntry();
		if (printOutput) {
			printOutputComposite = new PrintDepthOutputOptionsComposite(
					selectionGroup, SWT.NONE, false, installation);
		}
	}

	private void createVariablesEntry() {
		variablesComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		variablesComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		variablesComposite.setLayoutData(gridData);
		Label label = new Label(variablesComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Variables")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		variablesVarXRadio = new Button(variablesComposite, SWT.RADIO);
		variablesVar_123Radio = new Button(variablesComposite, SWT.RADIO);
		variablesVarX_123Radio = new Button(variablesComposite, SWT.RADIO);
		variablesVar_Radio = new Button(variablesComposite, SWT.RADIO);
		initialiseRadioComposite(variablesVarXRadio,
				PreferenceConstants.varXExample, "", variablesVar_123Radio,
				PreferenceConstants.var_123Example, "v",
				variablesVarX_123Radio, PreferenceConstants.varX_123Example,
				"V", variablesVar_Radio, PreferenceConstants.var_Example, "_");
	}

	private void createAttributesEntry() {
		attributesComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		attributesComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		attributesComposite.setLayoutData(gridData);
		Label label = new Label(attributesComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Attributes")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		attributesNoneRadio = new Button(attributesComposite, SWT.RADIO);
		attributesPrettyRadio = new Button(attributesComposite, SWT.RADIO);
		attributesFullRadio = new Button(attributesComposite, SWT.RADIO);
		initialiseRadioComposite(attributesNoneRadio, PreferenceMessages
				.getString("ECLiPSeStrings.None"), "", attributesPrettyRadio,
				PreferenceMessages.getString("ECLiPSeStrings.Pretty"), "m",
				attributesFullRadio, PreferenceMessages
						.getString("ECLiPSeStrings.Full"), "M", null, null,
				null);
	}

	private void createOperatorsEntry() {
		operatorsComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		operatorsComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		Label label = new Label(operatorsComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Operators")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		operatorsComposite.setLayoutData(gridData);
		operatorsInfixRadio = new Button(operatorsComposite, SWT.RADIO);
		operatorsPrefixRadio = new Button(operatorsComposite, SWT.RADIO);
		initialiseRadioComposite(operatorsInfixRadio,
				PreferenceConstants.infixOpExample, "", operatorsPrefixRadio,
				PreferenceConstants.prefixOpExample, "O", null, null, null,
				null, null, null);
	}

	private void createSpacesEntry() {
		spacesComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		spacesComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		spacesComposite.setLayoutData(gridData);
		Label label = new Label(spacesComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Spaces")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		spacesOnRadio = new Button(spacesComposite, SWT.RADIO);
		spacesOffRadio = new Button(spacesComposite, SWT.RADIO);
		initialiseRadioComposite(spacesOnRadio,
				PreferenceConstants.spaceExample, "", spacesOffRadio,
				PreferenceConstants.noSpaceExample, "K", null, null, null,
				null, null, null);
	}

	private void createQuotingEntry() {
		quotingComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		quotingComposite.setLayout(layout);
		Label label = new Label(quotingComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Quoting")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		quotingComposite.setLayoutData(gridData);
		quotingOffRadio = new Button(quotingComposite, SWT.RADIO);
		quotingOnRadio = new Button(quotingComposite, SWT.RADIO);
		initialiseRadioComposite(quotingOffRadio,
				PreferenceConstants.quotingExample, "", quotingOnRadio,
				PreferenceConstants.noQuotingExample, "Q", null, null, null,
				null, null, null);
	}

	private void createListsEntry() {
		listsComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		listsComposite.setLayout(layout);
		Label label = new Label(listsComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.Lists")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		listsComposite.setLayoutData(gridData);
		bracketListRadio = new Button(listsComposite, SWT.RADIO);
		dotListRadio = new Button(listsComposite, SWT.RADIO);
		initialiseRadioComposite(bracketListRadio,
				PreferenceConstants.bracketListExample, "", dotListRadio,
				PreferenceConstants.dotListExample, ".", null, null, null,
				null, null, null);
	}

	private void createPortrayEntry() {
		usePortrayComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		usePortrayComposite.setLayout(layout);
		Label label = new Label(usePortrayComposite, SWT.NONE);
		label.setText(PreferenceMessages.getString("ECLiPSeStrings.UsePortray")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		usePortrayComposite.setLayoutData(gridData);
		usePortrayOffRadio = new Button(usePortrayComposite, SWT.RADIO);
		usePortrayOnRadio = new Button(usePortrayComposite, SWT.RADIO);
		initialiseRadioComposite(usePortrayOffRadio, PreferenceMessages
				.getString("ECLiPSeStrings.No"), "", usePortrayOnRadio,
				PreferenceMessages.getString("ECLiPSeStrings.Yes"), "P", null,
				null, null, null, null, null);
	}

	private void createTransformationsEntry() {
		transformationsComposite = new Composite(selectionGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 5;
		transformationsComposite.setLayout(layout);
		Label label = new Label(transformationsComposite, SWT.NONE);
		label.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Transformations")
				+ ":");
		RadioCompositeBuilder.buildLabel(label);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		transformationsComposite.setLayoutData(gridData);
		transformationsOffRadio = new Button(transformationsComposite,
				SWT.RADIO);
		transformationsOnRadio = new Button(transformationsComposite, SWT.RADIO);
		initialiseRadioComposite(transformationsOffRadio, PreferenceMessages
				.getString("ECLiPSeStrings.No"), "T", transformationsOnRadio,
				PreferenceMessages.getString("ECLiPSeStrings.Yes"), "", null,
				null, null, null, null, null);
	}

	private void setWidgetValues(String outputModeString) {
		if (outputModeString.indexOf('_') != -1) {
			variablesVar_Radio.setSelection(true);
			variablesVarX_123Radio.setSelection(false);
			variablesVar_123Radio.setSelection(false);
			variablesVarXRadio.setSelection(false);
		} else if (outputModeString.indexOf('V') != -1) {
			variablesVar_Radio.setSelection(false);
			variablesVarX_123Radio.setSelection(true);
			variablesVar_123Radio.setSelection(false);
			variablesVarXRadio.setSelection(false);
		} else if (outputModeString.indexOf('v') != -1) {
			variablesVar_Radio.setSelection(false);
			variablesVarX_123Radio.setSelection(false);
			variablesVar_123Radio.setSelection(true);
			variablesVarXRadio.setSelection(false);
		} else {
			variablesVar_Radio.setSelection(false);
			variablesVarX_123Radio.setSelection(false);
			variablesVar_123Radio.setSelection(false);
			variablesVarXRadio.setSelection(true);
		}
		if (outputModeString.indexOf('M') != -1) {
			attributesFullRadio.setSelection(true);
			attributesPrettyRadio.setSelection(false);
			attributesNoneRadio.setSelection(false);
		} else if (outputModeString.indexOf('m') != -1) {
			attributesFullRadio.setSelection(false);
			attributesPrettyRadio.setSelection(true);
			attributesNoneRadio.setSelection(false);
		} else {
			attributesFullRadio.setSelection(false);
			attributesPrettyRadio.setSelection(false);
			attributesNoneRadio.setSelection(true);
		}
		if (outputModeString.indexOf('O') != -1) {
			operatorsInfixRadio.setSelection(false);
			operatorsPrefixRadio.setSelection(true);
		} else {
			operatorsInfixRadio.setSelection(true);
			operatorsPrefixRadio.setSelection(false);
		}
		if (outputModeString.indexOf('K') != -1) {
			spacesOnRadio.setSelection(false);
			spacesOffRadio.setSelection(true);
		} else {
			spacesOnRadio.setSelection(true);
			spacesOffRadio.setSelection(false);
		}
		if (outputModeString.indexOf('Q') != -1) {
			quotingOnRadio.setSelection(true);
			quotingOffRadio.setSelection(false);
		} else {
			quotingOnRadio.setSelection(false);
			quotingOffRadio.setSelection(true);
		}
		if (outputModeString.indexOf('.') != -1) {
			dotListRadio.setSelection(true);
			bracketListRadio.setSelection(false);
		} else {
			dotListRadio.setSelection(false);
			bracketListRadio.setSelection(true);
		}
		if (outputModeString.indexOf('P') != -1) {
			usePortrayOnRadio.setSelection(true);
			usePortrayOffRadio.setSelection(false);
		} else {
			usePortrayOnRadio.setSelection(false);
			usePortrayOffRadio.setSelection(true);
		}
		if (outputModeString.indexOf('T') != -1) {
			transformationsOnRadio.setSelection(false);
			transformationsOffRadio.setSelection(true);
		} else {
			transformationsOnRadio.setSelection(true);
			transformationsOffRadio.setSelection(false);
		}
	}

	public void setWidgetValues(EnginePreferences installation) {
		outputModeString = oldOutputModeString = installation
		.getStringFlag(PreferenceConstants.outputMode);
		setWidgetValues(oldOutputModeString);
	}

	/**
	 * This method initializes the radio button controls
	 */
	private void initialiseRadioComposite(Button option1, String text1,
			String data1, Button option2, String text2, String data2,
			Button option3, String text3, String data3, Button option4,
			String text4, String data4) {
		option1.setText(text1);
		option1.setData(data1);
		RadioCompositeBuilder.buildRadio(option1);
		radioButtons.add(option1);
		option2.setText(text2);
		option2.setData(data2);
		RadioCompositeBuilder.buildRadio(option2);
		radioButtons.add(option2);
		if (option3 != null) {
			option3.setText(text3);
			option3.setData(data3);
			radioButtons.add(option3);
		} else {
			option3 = new Button(option1.getParent(), SWT.RADIO);
			option3.setVisible(false);
		}
		if (option4 != null) {
			option4.setText(text4);
			option4.setData(data4);
			radioButtons.add(option4);
		} else {
			option4 = new Button(option1.getParent(), SWT.RADIO);
			option4.setVisible(false);
		}
		RadioCompositeBuilder.buildRadio(option3);
		RadioCompositeBuilder.buildRadio(option4);
	}

	/**
	 * This method initializes buttonComposite
	 * 
	 */
	private void createButtonComposite(boolean flag) {
		if (flag) {
			buttonComposite = new Composite(this, SWT.NONE);
			GridLayout layout = new GridLayout();
			layout.numColumns = 2;
			buttonComposite.setLayout(layout);
			GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gridData.grabExcessHorizontalSpace = true;
			buttonComposite.setLayoutData(gridData);
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL);
			gridData.grabExcessHorizontalSpace = true;
			gridData.grabExcessVerticalSpace = true;
			okButton = new Button(buttonComposite, SWT.PUSH);
			okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
			okButton.addSelectionListener(this);
			cancelButton = new Button(buttonComposite, SWT.PUSH);
			cancelButton.setText(PreferenceMessages
					.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
			cancelButton.setLayoutData(gridData);
			cancelButton.addSelectionListener(this);
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL);
			gridData.grabExcessHorizontalSpace = true;
			gridData.grabExcessVerticalSpace = true;
			okButton.setLayoutData(gridData);
		} /*
			 * else { GridLayout layout = new GridLayout();
			 * buttonComposite.setLayout(layout); GridData gridData = new
			 * GridData(GridData.HORIZONTAL_ALIGN_FILL);
			 * gridData.grabExcessHorizontalSpace = true;
			 * buttonComposite.setLayoutData(gridData); applyButton = new
			 * Button(buttonComposite, SWT.PUSH);
			 * applyButton.setText(PreferenceMessages
			 * .getString("ECLiPSeStrings.Apply")); //$NON-NLS-1$ gridData = new
			 * GridData(GridData.HORIZONTAL_ALIGN_END |
			 * GridData.VERTICAL_ALIGN_FILL); gridData.grabExcessHorizontalSpace =
			 * true; gridData.grabExcessVerticalSpace = true;
			 * gridData.minimumWidth = 3 * applyButton.computeSize(SWT.DEFAULT,
			 * SWT.DEFAULT).x; applyButton.setLayoutData(gridData);
			 * applyButton.addSelectionListener(new SelectionAdapter() { public
			 * void widgetSelected(SelectionEvent e) { setNewValues(); } }); }
			 */
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == okButton) {
			setOutputModeString(true);
			parentComposite.dispose();
		} else if (event.getSource() == cancelButton) {
			outputModeString = oldOutputModeString;
			parentComposite.dispose();
		}
	}

	public String getOutputModeString() {
		return outputModeString;
	}

	public void setOutputModeString(boolean addFullPrintDepth) {
		outputModeString = "";

		for (int cnt = 0; cnt < radioButtons.size(); cnt++) {
			Button radioButton = (Button) radioButtons.get(cnt);
			if (radioButton.getSelection()) {
				outputModeString = outputModeString.concat((String) radioButton
						.getData());
			}
		}

		if (addFullPrintDepth) {
			// Add in the fullDepthChar if output_mode is overriding print_depth
			if (oldOutputModeString.indexOf(PreferenceConstants.fullDepthChar) != -1) {
				outputModeString += PreferenceConstants.fullDepthChar;
			}
		}

		oldOutputModeString = outputModeString;
	}

	public void setEnabled(boolean enabled) {
		if (printOutput) {
			printOutputComposite.setEnabled(enabled);
		}
		selectionGroup.setEnabled(enabled);
		variablesVarXRadio.setEnabled(enabled);
		variablesVar_123Radio.setEnabled(enabled);
		variablesVarX_123Radio.setEnabled(enabled);
		variablesVar_Radio.setEnabled(enabled);
		attributesNoneRadio.setEnabled(enabled);
		attributesPrettyRadio.setEnabled(enabled);
		attributesFullRadio.setEnabled(enabled);
		operatorsInfixRadio.setEnabled(enabled);
		operatorsPrefixRadio.setEnabled(enabled);
		spacesOnRadio.setEnabled(enabled);
		spacesOffRadio.setEnabled(enabled);
		quotingOffRadio.setEnabled(enabled);
		quotingOnRadio.setEnabled(enabled);
		bracketListRadio.setEnabled(enabled);
		dotListRadio.setEnabled(enabled);
		usePortrayOffRadio.setEnabled(enabled);
		usePortrayOnRadio.setEnabled(enabled);
		transformationsOffRadio.setEnabled(enabled);
		transformationsOnRadio.setEnabled(enabled);
	}

	private String oldOutputModeString = null;

	private String outputModeString = null;

	private PrintDepthOutputOptionsComposite printOutputComposite = null;

	private boolean printOutput = false;

	private Vector radioButtons = null;

	private Group selectionGroup = null;

	private Composite variablesComposite = null;

	private Composite attributesComposite = null;

	private Composite operatorsComposite = null;

	private Composite spacesComposite = null;

	private Composite quotingComposite = null;

	private Composite listsComposite = null;

	private Composite usePortrayComposite = null;

	private Composite transformationsComposite = null;

	private Composite buttonComposite = null;

	private Button okButton = null;

	private Button cancelButton = null;

	private Composite parentComposite = null;

	private Button variablesVarXRadio = null;

	private Button variablesVar_123Radio = null;

	private Button variablesVarX_123Radio = null;

	private Button variablesVar_Radio = null;

	private Button attributesNoneRadio = null;

	private Button attributesPrettyRadio = null;

	private Button attributesFullRadio = null;

	private Button operatorsInfixRadio = null;

	private Button operatorsPrefixRadio = null;

	private Button spacesOnRadio = null;

	private Button spacesOffRadio = null;

	private Button quotingOffRadio = null;

	private Button quotingOnRadio = null;

	private Button bracketListRadio = null;

	private Button dotListRadio = null;

	private Button usePortrayOffRadio = null;

	private Button usePortrayOnRadio = null;

	private Button transformationsOffRadio = null;

	private Button transformationsOnRadio = null;
}
