/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.*;

public class ECLiPSeOutputOptionsShell {

	private EnginePreferences installation = null;

	public ECLiPSeOutputOptionsShell(EnginePreferences installation) {
		this.installation = installation;
	}

	/**
	 * This method initializes OutputOptionsShell
	 */
	private void createOutputOptionsShell(Shell parent) {
		outputOptionsShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		outputOptionsShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSe")
				+ PreferenceMessages.getString("ECLiPSeStrings.OutputOptions")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		outputOptionsShell.setLayout(layout);
		outputOptions = new ECLiPSeOutputOptions(outputOptionsShell, SWT.NONE,
				false, true, installation);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		outputOptions.setLayoutData(gridData);
		outputOptionsShell.setSize(outputOptionsShell.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
	}

	/**
	 * @return true if shell is disposed
	 */
	public boolean isDisposed() {
		return outputOptionsShell.isDisposed();
	}

	/**
	 * this method opens the output options shell
	 * 
	 * @param parent
	 * @return
	 */
	public String open(Shell parent) {
		createOutputOptionsShell(parent);
		Point parentSize = parent.getSize();
		Point size = outputOptionsShell.getSize();
		int x = (parentSize.x - size.x) / 2;
		int y = (parentSize.y - size.y) / 2;
		outputOptionsShell.setLocation(parent.toDisplay(x, y));
		outputOptionsShell.open();
		Display display = Display.getDefault();
		while (!outputOptionsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return ((ECLiPSeOutputOptions) outputOptions).getOutputModeString();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order
		 * to run with the SWT dlls. The dlls are located in the SWT plugin jar.
		 * For example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */
		Display display = Display.getDefault();
		ECLiPSeOutputOptionsShell thisClass = new ECLiPSeOutputOptionsShell(
				null);
		thisClass.open(null);

		while (!thisClass.outputOptionsShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}

	private Shell outputOptionsShell = null; // @jve:decl-index=0:visual-constraint="10,10"

	private Composite outputOptions = null;

}
