/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;

public class EngineInstallationSelector extends Composite implements
		SelectionListener {

	public EngineInstallationSelector(Composite parent, int style,
			boolean showLink) {
		super(parent, style);
		this.showLink = showLink;
		initialise();
	}

	private void initialise() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		GridLayout layout = new GridLayout();
		this.setLayout(layout);
		this.setLayoutData(gridData);
		createEngineGroup();
		setWidgetValues();
		this.setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createEngineGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		layout.verticalSpacing = 8;
		layout.numColumns = 3;
		engineInstallationGroup = new Group(this, SWT.NONE);
		engineInstallationGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngine")); //$NON-NLS-1$
		engineInstallationGroup.setLayout(layout);
		engineInstallationGroup.setLayoutData(gridData);
		defaultInstallationRadio = new Button(engineInstallationGroup,
				SWT.RADIO);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		if (showLink) {
			gridData.horizontalSpan = 2;
			installationLink = new Link(engineInstallationGroup, SWT.NONE);
			installationLink.setText("<A>"
					+ PreferenceMessages
							.getString("ECLiPSeStrings.ConfigureInstallations")
					+ "...</A>");
			installationLink.addSelectionListener(this);
		} else {
			gridData.horizontalSpan = 3;
		}
		defaultInstallationRadio.setLayoutData(gridData);
		defaultInstallationRadio.addSelectionListener(this);
		specificInstallationRadio = new Button(engineInstallationGroup,
				SWT.RADIO);
		specificInstallationRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.UseProjectSpecificInstallation")
				+ ":");
		specificInstallationRadio.addSelectionListener(this);
		specificInstallationCombo = new Combo(engineInstallationGroup,
				SWT.DROP_DOWN | SWT.READ_ONLY);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		specificInstallationCombo.setLayoutData(gridData);
		specificInstallationCombo.setEnabled(false);
		specificInstallationCombo.setBackground(Display.getDefault()
				.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
	}

	private void setWidgetValues() {
		String defaultInstallationName = PreferenceManager
				.getDefaultInstallationName();
		if (defaultInstallationName.equals("")) {
			defaultInstallationRadio.setText(PreferenceMessages
					.getString("ECLiPSeStrings.UseDefaultInstallation")
					+ " "
					+ PreferenceMessages.getString("ECLiPSeStrings.Undefined")
					+ ")");

		} else {
			defaultInstallationRadio.setText(PreferenceMessages
					.getString("ECLiPSeStrings.UseDefaultInstallation")
					+ " '" + defaultInstallationName + "')");
		}
		defaultInstallationRadio.setSelection(true);
		specificInstallationRadio.setSelection(false);
		specificInstallationCombo.setEnabled(false);
		specificInstallationCombo.setItems(PreferenceManager
				.getInstallationNames());
		specificInstallationCombo.setText(defaultInstallationName);
	}

	public void setSpecificInstallation(String installation) {
		defaultInstallationRadio.setSelection(false);
		specificInstallationRadio.setSelection(true);
		specificInstallationCombo.setEnabled(true);
		specificInstallationCombo.setText(installation);

	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == defaultInstallationRadio) {
			defaultInstallationRadio.setSelection(true);
			specificInstallationRadio.setSelection(false);
			specificInstallationCombo.setEnabled(false);
			if (showLink) {
				installationLink.setEnabled(true);
			}
		} else if (event.getSource() == specificInstallationRadio) {
			defaultInstallationRadio.setSelection(false);
			specificInstallationRadio.setSelection(true);
			specificInstallationCombo.setEnabled(true);
			if (showLink) {
				installationLink.setEnabled(false);
			}
		} else if (event.getSource() == installationLink) {
			configureWorkspacePreferences();
		}
	}

	public void configureWorkspacePreferences() {
		PreferencesUtil.createPreferenceDialogOn(
				getShell(),
				PreferenceConstants.EngineInstallationPreferencePageId,
				new String[] {
						PreferenceConstants.EngineInstallationPreferencePageId,
						PreferenceConstants.EngineSettingsPreferencePageId,
						PreferenceConstants.GCPreferencePageId,
						PreferenceConstants.OutputOptionsPreferencePageId },
				null).open();
		setWidgetValues();
	}

	public void commit() {
		projectSpecificInstallation = specificInstallationRadio.getSelection();

		if (projectSpecificInstallation) {
			installationName = specificInstallationCombo.getText();
		} else {
			installationName = "";
		}
	}

	public void revertToDefaults() {
		defaultInstallationRadio.setSelection(true);
		specificInstallationRadio.setSelection(false);
		specificInstallationCombo.setEnabled(false);
		if (showLink) {
			installationLink.setEnabled(true);
		}
	}

	public boolean isProjectSpecificInstallation() {
		return projectSpecificInstallation;
	}

	public String getInstallationName() {
		return installationName;
	}

	public String[] getInstallationNames() {
		return specificInstallationCombo.getItems();
	}

	public void setEnabled(boolean enabled) {
		engineInstallationGroup.setEnabled(enabled);
		if (showLink) {
			installationLink.setEnabled(enabled);
		}
		if (enabled) {
			if (specificInstallationRadio.getSelection()) {
				specificInstallationCombo.setEnabled(true);
			}
		} else {
			specificInstallationCombo.setEnabled(false);
		}
		defaultInstallationRadio.setEnabled(enabled);
		specificInstallationRadio.setEnabled(enabled);
	}

	private Group engineInstallationGroup = null;

	private Button defaultInstallationRadio = null;

	private Button specificInstallationRadio = null;

	private Combo specificInstallationCombo = null;

	private Link installationLink = null;

	private String installationName = "";

	private boolean projectSpecificInstallation = false;

	private boolean showLink = false;
}
