/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.core.preferences;

import java.util.Hashtable;

public class EnginePreferences implements Cloneable {

	public final static int ATOM = 0;

	public final static int STRING = 1;

	public final static int INTEGER = 2;

	public final static int LIST = 3;

	public static String[] flagNames = { PreferenceConstants.afterEventTimer,
			PreferenceConstants.allDynamic,
			PreferenceConstants.brealExceptions, PreferenceConstants.coroutine,
			PreferenceConstants.debugging, PreferenceConstants.debugCompile,
			PreferenceConstants.enableInterrupts,
			PreferenceConstants.goalExpansion,
			PreferenceConstants.macroExpansion,
			PreferenceConstants.preferRationals,
			PreferenceConstants.variableNames, PreferenceConstants.outputMode,
			PreferenceConstants.printDepth, PreferenceConstants.gc,
			PreferenceConstants.gcPolicy, PreferenceConstants.gcInterval,
			PreferenceConstants.gcIntervalDict };

	public static int[] flagTypes = { ATOM, ATOM, ATOM, ATOM, ATOM, ATOM, ATOM,
			ATOM, ATOM, ATOM, ATOM, STRING, INTEGER, ATOM, ATOM, INTEGER,
			INTEGER };

	public EnginePreferences() {
		flags = new Hashtable();
	}

	public EnginePreferences(String installationName, String installationPath,
			String libraryPath, String engineArguments) {
		flags = new Hashtable();
		this.installationName = installationName;
		this.installationPath = installationPath;
		this.libraryPath = libraryPath;
		this.engineArguments = engineArguments;
	}

	public String getEngineArguments() {
		return engineArguments;
	}

	public void setEngineArguments(String engineArguments) {
		this.engineArguments = engineArguments;
	}

	public String getInstallationName() {
		return installationName;
	}

	public void setInstallationName(String installationName) {
		this.installationName = installationName;
	}

	public String getInstallationPath() {
		return installationPath;
	}

	public void setInstallationPath(String installationPath) {
		this.installationPath = installationPath;
	}

	public String getLibraryPath() {
		return libraryPath;
	}

	public void setLibraryPath(String libraryPath) {
		this.libraryPath = libraryPath;
	}

	public String getInstallationVersion() {
		return installationVersion;
	}

	public void setInstallationVersion(String installationVersion) {
		this.installationVersion = installationVersion;
	}

	public void setFlag(String flag, Object value) {
		flags.put(flag, value);
	}

	public Object getFlag(String flag) {
		return flags.get(flag);
	}

	public String getStringFlag(String flag) {
		String value = (String) flags.get(flag);
		if (value == null) {
			return "";
		}

		return value;
	}

	public int getIntFlag(String flag) {
		Integer value = (Integer) flags.get(flag);
		if (value == null) {
			return -1;
		}

		return value.intValue();
	}

	public Hashtable getFlags() {
		return flags;
	}

	public void setFlags(Hashtable flags) {
		this.flags = flags;
	}

	public Object clone() {
		EnginePreferences clone = new EnginePreferences(new String(
				installationName), new String(installationPath), new String(
				libraryPath), new String(engineArguments));
		clone.setInstallationVersion(new String(installationVersion));

		for (int flag = 0; flag < flagNames.length; flag++) {
			switch (flagTypes[flag]) {
			case ATOM:
			case STRING:
				clone.setFlag(flagNames[flag], new String((String) flags
						.get(flagNames[flag])));
				break;
			case INTEGER:
				clone.setFlag(flagNames[flag], new Integer(((Integer) flags
						.get(flagNames[flag])).intValue()));
				break;
			}
		}

		return clone;
	}

	public boolean equals(Object o) {
		// EnginePreferences ep = (EnginePreferences) o;
		// System.err.println(installationName + " " + ep.installationName);
		// System.err.println(installationPath + " " + ep.installationPath);
		// System.err.println(libraryPath + " " + ep.libraryPath);
		// System.err.println(engineArguments + " " + ep.engineArguments);
		// System.err.println(installationVersion + " " +
		// ep.installationVersion);
		// System.err.println(flags.equals(((EnginePreferences) o).flags));
		// for (Enumeration e = flags.keys(); e.hasMoreElements();) {
		// String key = (String) e.nextElement();
		// System.err.println(key + ": " + flags.get(key) + " "
		// + ep.flags.get(key));
		// }
		// Don't need to test flags for null!
		boolean result = (o != null)
				&& (this.getClass() == o.getClass())
				&& (installationName == ((EnginePreferences) o).installationName || (installationName != null && installationName
						.equals(((EnginePreferences) o).installationName)))
				&& (installationPath == ((EnginePreferences) o).installationPath || (installationPath != null && installationPath
						.equals(((EnginePreferences) o).installationPath)))
				&& (libraryPath == ((EnginePreferences) o).libraryPath || (libraryPath != null && libraryPath
						.equals(((EnginePreferences) o).libraryPath)))
				&& (engineArguments == ((EnginePreferences) o).engineArguments || (engineArguments != null && engineArguments
						.equals(((EnginePreferences) o).engineArguments)))
				&& (installationVersion == ((EnginePreferences) o).installationVersion || (installationVersion != null && installationVersion
						.equals(((EnginePreferences) o).installationVersion)));

		if (!result) {
			return false;
		}

		if (!flags.equals(((EnginePreferences) o).flags)) {
			String output_mode = (String) flags
					.get(PreferenceConstants.outputMode);
			String output_mode1 = (String) ((EnginePreferences) o).flags
					.get(PreferenceConstants.outputMode);
			if (output_mode.length() != output_mode1.length()) {
				return false;
			}
			for (int c = 0; c < output_mode.length(); c++) {
				if (output_mode.indexOf(output_mode1.charAt(c)) == -1) {
					return false;
				}
			}
		}
		return true;
	}

	public int hashCode() {
		int result = 17;
		if (installationName != null) {
			result = 37 + result * installationName.hashCode();
		}

		if (installationPath != null) {
			result = 37 + result * installationPath.hashCode();
		}

		if (installationVersion != null) {
			result = 37 + result * installationVersion.hashCode();
		}

		if (engineArguments != null) {
			result = 37 + result * engineArguments.hashCode();
		}

		if (installationVersion != null) {
			result = 37 + result * installationVersion.hashCode();
		}

		result = 37 + result * flags.hashCode();

		return result;
	}

	private String installationName = "";

	private String installationPath = "";

	private String libraryPath = "";

	private String engineArguments = "";

	private String installationVersion = "";

	private Hashtable flags = null;
}
