/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class ECLiPSeFilterConfiguration extends Composite {

	// widgets
	private Group invocationGroup = null;

	private Group portTypesGroup = null;

	private Group predSpecificationGroup = null;

	private Group condCountGroup = null;

	private Label invocationLabel = null;

	private Label invocationToLabel = null;

	private Label depthLabel = null;

	private Label depthToLabel = null;

	private Text invocationFromText = null;

	private Text invocationToText = null;

	private Text depthFromText = null;

	private Text depthToText = null;

	private Button anyButton = null;

	private Button anySpyButton = null;

	private Button specificButton = null;

	private Label definingLabel = null;

	private Combo definingCombo = null;

	private Label templateLabel = null;

	private Text templateText = null;

	private Label conditionLabel = null;

	private Text conditionText = null;

	private Label callingLabel = null;

	private Combo callingCombo = null;

	private Label condCounterLabel = null;

	private Label condCounterStopLabel = null;

	private Label condCounterTimesLabel = null;

	private Text condCounterText = null;

	private Button cancelButton = null;

	private Button okButton = null;

	private Composite buttonComposite = null;

	private Composite parentComposite = null;

	public ECLiPSeFilterConfiguration(Composite parent, int style,
			ECLiPSeInteractionInterface eclipseInteraction, ECLiPSeFilter filter) {
		super(parent, style);
		parentComposite = parent;
		this.eclipseInteraction = eclipseInteraction;
		this.oldFilter = filter;
		try {
			this.filter = (ECLiPSeFilter) filter.clone();
		} catch (CloneNotSupportedException e) {
			System.err
					.println("ECLiPSeFilter class contains an uncloneable object. Please report this as a bug.");
			e.printStackTrace();
		}
		initialize();
	}

	/**
	 * This method initializes the ECLiPSeFilterConfiguration widget
	 * 
	 */
	private void initialize() {
		GridLayout gridLayout = new GridLayout();
		this.setLayout(gridLayout);
		createInvocationGroup();
		createPortTypesGroup();
		createPredSpecificationGroup();
		createCondCountGroup();
		createButtonComposite();
		setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * This method initializes invocationGroup
	 * 
	 */
	private void createInvocationGroup() {
		invocationGroup = new Group(this, SWT.NONE);
		invocationGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InvocationAndDepth")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		invocationGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		invocationGroup.setLayoutData(gridData);
		invocationLabel = new Label(invocationGroup, SWT.NONE);
		invocationLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InvocationNumberFrom") + ":"); //$NON-NLS-1$ //$NON-NLS-2$
		invocationFromText = new Text(invocationGroup, SWT.BORDER);
		invocationFromText.setText("" + filter.getFromInvocationFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		invocationFromText.setLayoutData(gridData);
		invocationToLabel = new Label(invocationGroup, SWT.NONE);
		invocationToLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.To")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		invocationToText = new Text(invocationGroup, SWT.BORDER);
		invocationToText.setText("" + filter.getToInvocationFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		invocationToText.setLayoutData(gridData);
		depthLabel = new Label(invocationGroup, SWT.NONE);
		depthLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.DepthFrom") + ":"); //$NON-NLS-1$ //$NON-NLS-2$
		depthFromText = new Text(invocationGroup, SWT.BORDER);
		depthFromText.setText("" + filter.getFromDepthFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.minimumWidth = invocationToText.computeSize(SWT.DEFAULT,
				SWT.DEFAULT).x;
		depthFromText.setLayoutData(gridData);
		depthToLabel = new Label(invocationGroup, SWT.NONE);
		depthToLabel.setText(PreferenceMessages.getString("ECLiPSeStrings.To")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		depthToText = new Text(invocationGroup, SWT.BORDER);
		depthToText.setText("" + filter.getToDepthFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		depthToText.setLayoutData(gridData);
	}

	/**
	 * This method initializes portTypesGroup
	 * 
	 */
	private void createPortTypesGroup() {
		portTypesGroup = new Group(this, SWT.NONE);
		portTypesGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.PortTypes")); //$NON-NLS-1$
		callPortButton = new Button(portTypesGroup, SWT.CHECK);
		callPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.call"));
		callPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		callPortButton.setSelection(filter.isCallPortFilter());
		exitPortButton = new Button(portTypesGroup, SWT.CHECK);
		exitPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.exit"));
		exitPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		exitPortButton.setSelection(filter.isExitPortFilter());
		starExitPortButton = new Button(portTypesGroup, SWT.CHECK);
		starExitPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.*exit"));
		starExitPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		starExitPortButton.setSelection(filter.isStarExitPortFilter());
		redoPortButton = new Button(portTypesGroup, SWT.CHECK);
		redoPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.redo"));
		redoPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		redoPortButton.setSelection(filter.isRedoPortFilter());
		failPortButton = new Button(portTypesGroup, SWT.CHECK);
		failPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.fail"));
		failPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		failPortButton.setSelection(filter.isFailPortFilter());
		resumePortButton = new Button(portTypesGroup, SWT.CHECK);
		resumePortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.resume"));
		resumePortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		resumePortButton.setSelection(filter.isResumePortFilter());
		leavePortButton = new Button(portTypesGroup, SWT.CHECK);
		leavePortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.leave"));
		leavePortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		leavePortButton.setSelection(filter.isLeavePortFilter());
		delayPortButton = new Button(portTypesGroup, SWT.CHECK);
		delayPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.delay"));
		delayPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		delayPortButton.setSelection(filter.isDelayPortFilter());
		nextPortButton = new Button(portTypesGroup, SWT.CHECK);
		nextPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.next"));
		nextPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		nextPortButton.setSelection(filter.isNextPortFilter());
		unifyPortButton = new Button(portTypesGroup, SWT.CHECK);
		unifyPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.unify"));
		unifyPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		unifyPortButton.setSelection(filter.isUnifyPortFilter());
		spytermPortButton = new Button(portTypesGroup, SWT.CHECK);
		spytermPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.spyterm"));
		spytermPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		spytermPortButton.setSelection(filter.isSpytermPortFilter());
		modifyPortButton = new Button(portTypesGroup, SWT.CHECK);
		modifyPortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.modify"));
		modifyPortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		modifyPortButton.setSelection(filter.isModifyPortFilter());
		elsePortButton = new Button(portTypesGroup, SWT.CHECK);
		elsePortButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.else"));
		elsePortButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true,
				false));
		elsePortButton.setSelection(filter.isElsePortFilter());
		GridLayout layout = new GridLayout();
		layout.numColumns = 7;
		portTypesGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		portTypesGroup.setLayoutData(gridData);
	}

	/**
	 * This method initializes predSpecificationGroup
	 * 
	 */
	private void createPredSpecificationGroup() {
		predSpecificationGroup = new Group(this, SWT.NONE);
		predSpecificationGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.PredicateSpecification")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		predSpecificationGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		predSpecificationGroup.setLayoutData(gridData);
		anyButton = new Button(predSpecificationGroup, SWT.RADIO);
		anyButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.AnyPredicate")); //$NON-NLS-1$
		anyButton.setSelection(filter.isAnyPredicateFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		anyButton.setLayoutData(gridData);
		anySpyButton = new Button(predSpecificationGroup, SWT.RADIO);
		anySpyButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.AnyPredicateWithSpypoint")); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		anySpyButton.setLayoutData(gridData);
		anySpyButton.setSelection(filter.isAnySpiedPredicateFilter());
		specificButton = new Button(predSpecificationGroup, SWT.RADIO);
		specificButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SpecificPredicateInstance")); //$NON-NLS-1$
		specificButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				boolean selected = specificButton.getSelection();
				definingLabel.setEnabled(selected);
				definingCombo.setEnabled(selected);
				templateLabel.setEnabled(selected);
				templateText.setEnabled(selected);
				conditionLabel.setEnabled(selected);
				conditionText.setEnabled(selected);
				callingLabel.setEnabled(selected);
				callingCombo.setEnabled(selected);
				getModuleListForCombos();
			}
		});
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		specificButton.setLayoutData(gridData);
		specificButton.setSelection(filter.isSpecificPredicateInstanceFilter());
		definingLabel = new Label(predSpecificationGroup, SWT.NONE);
		definingLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.DefiningModule") + ":"); //$NON-NLS-1$
		definingLabel.setEnabled(specificButton.getSelection());
		definingCombo = new Combo(predSpecificationGroup, SWT.BORDER
				| SWT.READ_ONLY);
		definingCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		definingCombo.setEnabled(specificButton.getSelection());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		definingCombo.setLayoutData(gridData);
		templateLabel = new Label(predSpecificationGroup, SWT.NONE);
		templateLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.GoalTemplate") + ":"); //$NON-NLS-1$
		templateLabel.setEnabled(specificButton.getSelection());
		templateText = new Text(predSpecificationGroup, SWT.BORDER);
		templateText.setText(filter.getGoalTemplateFilter());
		templateText.setEnabled(specificButton.getSelection());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		templateText.setLayoutData(gridData);
		conditionLabel = new Label(predSpecificationGroup, SWT.NONE);
		conditionLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Condition") + ":"); //$NON-NLS-1$
		conditionLabel.setEnabled(specificButton.getSelection());
		conditionText = new Text(predSpecificationGroup, SWT.BORDER);
		conditionText.setText(filter.getConditionFilter());
		conditionText.setEnabled(specificButton.getSelection());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		conditionText.setLayoutData(gridData);
		callingLabel = new Label(predSpecificationGroup, SWT.NONE);
		callingLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CallingModule") + ":"); //$NON-NLS-1$
		callingLabel.setEnabled(specificButton.getSelection());
		callingCombo = new Combo(predSpecificationGroup, SWT.BORDER
				| SWT.READ_ONLY);
		callingCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		callingCombo.setEnabled(specificButton.getSelection());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		callingCombo.setLayoutData(gridData);
		getModuleListForCombos();
	}

	/**
	 * This method initializes condCountGroup
	 * 
	 */
	private void createCondCountGroup() {
		condCountGroup = new Group(this, SWT.NONE);
		condCountGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ConditionCounter")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		condCountGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		condCountGroup.setLayoutData(gridData);
		condCounterStopLabel = new Label(condCountGroup, SWT.NONE);
		condCounterStopLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.StopAfterConditionsMet")); //$NON-NLS-1$
		condCounterText = new Text(condCountGroup, SWT.BORDER);
		condCounterText.setText("" + filter.getConditionCounterFilter());
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		condCounterText.setLayoutData(gridData);
		condCounterTimesLabel = new Label(condCountGroup, SWT.NONE);
		condCounterTimesLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Times") + "."); //$NON-NLS-1$
		condCounterLabel = new Label(condCountGroup, SWT.NONE);
		condCounterLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ConditionsMet")
				+ " "
				+ +filter.getConditionMetCounter()
				+ " "
				+ PreferenceMessages
						.getString("ECLiPSeStrings.TimesUsingFilter") + ".");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 3;
		condCounterLabel.setLayoutData(gridData);
	}

	/**
	 * This method constructs the Cancel/Apply/OK button group
	 * 
	 */
	private void createButtonComposite() {
		buttonComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		buttonComposite.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		buttonComposite.setLayoutData(gridData);
		okButton = new Button(buttonComposite, SWT.PUSH);
		okButton.setText(PreferenceMessages.getString("ECLiPSeStrings.OK")); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		okButton.setLayoutData(gridData);
		okButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setFilterValues();
				if (oldFilter.equals(filter)) {
					filter = oldFilter;
				} else {
					filter.setModified(true);
				}
				parentComposite.dispose();
			}
		});
		cancelButton = new Button(buttonComposite, SWT.PUSH);
		cancelButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Cancel")); //$NON-NLS-1$
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		cancelButton.setLayoutData(gridData);
		cancelButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				filter = oldFilter;
				parentComposite.dispose();
			}
		});
	}

	private void getModuleListForCombos() {
		CompoundTerm command = new CompoundTermImpl(":",
				new Atom("tracer_tcl"), new CompoundTermImpl("list_modules",
						null, null));
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				ArrayList moduleList = new ArrayList(
						(Collection) ((CompoundTerm) getResult().arg(2)).arg(1));
				String[] modules = new String[moduleList.size()];
				for (int index = 0; index < moduleList.size(); index++) {
					modules[index] = ((Atom) moduleList.get(index)).functor()
							.toString();
				}
				definingCombo.setItems(modules);
				callingCombo.setItems(modules);
				callingCombo.add("", 0);
				String definingModule = filter.getDefiningModuleFilter();
				if (definingModule.equals("")) {
					String toplevelModule = ((Atom) ((CompoundTerm) getResult()
							.arg(2)).arg(2)).functor();
					definingCombo.setText(toplevelModule);
				} else {
					definingCombo.setText(definingModule);
				}
				callingCombo.setText(filter.getCallingModuleFilter());
			}
		};
		eclipseInteraction.interruptingRpc(command,
				"Failed to refresh tracer filter module list.", r, null);
	}

	private void setFilterValues() {
		filter.setAnyPredicateFilter(anyButton.getSelection());
		filter.setAnySpiedPredicateFilter(anySpyButton.getSelection());
		filter.setCallPortFilter(callPortButton.getSelection());
		filter.setConditionCounterFilter(new Integer(condCounterText.getText())
				.intValue());
		filter.setDelayPortFilter(delayPortButton.getSelection());
		filter.setElsePortFilter(elsePortButton.getSelection());
		filter.setExitPortFilter(exitPortButton.getSelection());
		filter.setFailPortFilter(failPortButton.getSelection());
		filter.setFromDepthFilter(new Integer(depthFromText.getText())
				.intValue());
		filter
				.setFromInvocationFilter(new Integer(invocationFromText
						.getText()).intValue());
		filter.setLeavePortFilter(leavePortButton.getSelection());
		filter.setModifyPortFilter(modifyPortButton.getSelection());
		filter.setNextPortFilter(nextPortButton.getSelection());
		filter.setRedoPortFilter(redoPortButton.getSelection());
		filter.setResumePortFilter(resumePortButton.getSelection());
		filter.setSpytermPortFilter(spytermPortButton.getSelection());
		filter.setStarExitPortFilter(starExitPortButton.getSelection());
		filter.setToDepthFilter(new Integer(depthToText.getText()).intValue());
		filter.setToInvocationFilter(new Integer(invocationToText.getText())
				.intValue());
		filter.setUnifyPortFilter(unifyPortButton.getSelection());
		boolean specific = specificButton.getSelection();
		filter.setSpecificPredicateInstanceFilter(specific);
		if (specific) {
			filter.setCallingModuleFilter(callingCombo.getText());
			filter.setDefiningModuleFilter(definingCombo.getText());
			filter.setGoalTemplateFilter(templateText.getText());
			filter.setConditionFilter(conditionText.getText());
		} else {
			filter.setCallingModuleFilter("");
			filter.setDefiningModuleFilter("");
			filter.setGoalTemplateFilter("");
			filter.setConditionFilter("");
		}
	}

	public ECLiPSeFilter getFilter() {
		return filter;
	}

	private Button callPortButton = null;

	private Button exitPortButton = null;

	private Button starExitPortButton = null;

	private Button redoPortButton = null;

	private Button failPortButton = null;

	private Button resumePortButton = null;

	private Button leavePortButton = null;

	private Button delayPortButton = null;

	private Button nextPortButton = null;

	private Button unifyPortButton = null;

	private Button spytermPortButton = null;

	private Button modifyPortButton = null;

	private Button elsePortButton = null;

	private ECLiPSeFilter oldFilter = null;

	private ECLiPSeFilter filter = null;

	private ECLiPSeInteractionInterface eclipseInteraction = null;
}
