/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.*;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.ECLiPSeFilterConfiguration;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

public class ECLiPSeFilterConfigurationShell {

	public ECLiPSeFilterConfigurationShell(
			ECLiPSeInteractionInterface eclipseInteraction, ECLiPSeFilter filter) {
		this.eclipseInteraction = eclipseInteraction;
		this.filter = filter;
	}

	private void createFilterConfigurationShell(Shell parent) {
		filterConfigurationShell = new Shell(parent, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		filterConfigurationShell.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeFilterConfiguration")); //$NON-NLS-1$
		GridLayout layout = new GridLayout();
		filterConfigurationShell.setLayout(layout);
		filterConfiguration = new ECLiPSeFilterConfiguration(
				filterConfigurationShell, SWT.NONE, eclipseInteraction, filter);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		filterConfiguration.setLayoutData(gridData);
		filterConfigurationShell.setSize(filterConfigurationShell.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));
	}

	public ECLiPSeFilter open(Shell parent) {
		createFilterConfigurationShell(parent);
		Point parentSize = parent.getSize();
		Point size = filterConfiguration.getSize();
		int x = (parentSize.x - size.x) / 2;
		int y = (parentSize.y - size.y) / 2;
		filterConfigurationShell.setLocation(parent.toDisplay(x, y));
		filterConfigurationShell.open();
		// block display until shell is closed
		Display display = Display.getDefault();
		while (!filterConfigurationShell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return filterConfiguration.getFilter();
	}

	private Shell filterConfigurationShell = null;

	private ECLiPSeFilterConfiguration filterConfiguration = null;

	private ECLiPSeFilter filter = null;

	private ECLiPSeInteractionInterface eclipseInteraction = null;
}
