package org.eclipseclp.saros.development.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.development.builders.SarosNature;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironmentEditorInput;
import org.eclipseclp.saros.development.editors.ECLiPSeSourceEditor;

public class LauncherAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;

	/**
	 * The constructor.
	 */
	public LauncherAction() {
	}

	public void run(IAction action) {
		IProject project = null;
		IWorkbenchPage page = window.getActivePage();
		if (page == null) {
			return;
		}

		if (PreferenceManager.launchActionSelection() && selection != null
				&& selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				if (obj instanceof IFile) {
					try {
						window.getActivePage().openEditor(
								new ECLiPSeExecutionEnvironmentEditorInput(
										(IResource) obj),
								ECLiPSeExecutionEnvironment.ID);
					} catch (PartInitException e) {
						Status status = new Status(
								IStatus.ERROR,
								"Saros",
								0,
								"Exception caught whilst launching execution environment: ",
								e);
						ErrorDialog.openError(window.getShell(),
								"Saros Development Plugin Error", e
										.getMessage(), status);
					}
					return;
				} else if (obj instanceof IProject) {
					project = (IProject) obj;
				}
			}
		}

		if (project == null) {
			IEditorPart part = window.getActivePage().getActiveEditor();
			if ((part instanceof ECLiPSeSourceEditor)) {
				IFile file = ((FileEditorInput) ((ECLiPSeSourceEditor) part)
						.getEditorInput()).getFile();
				try {
					window.getActivePage().openEditor(
							new ECLiPSeExecutionEnvironmentEditorInput(file, false, true),
							ECLiPSeExecutionEnvironment.ID);
				} catch (PartInitException e) {
					Status status = new Status(
							IStatus.ERROR,
							"Saros",
							0,
							"Exception caught whilst launching execution environment: ",
							e);
					ErrorDialog.openError(window.getShell(),
							"Saros Development Plugin Error", e.getMessage(),
							status);
				}
				return;
			}
		}

		try {
			if (project != null
					&& project.getNature(SarosNature.NATURE_ID) == null) {
				project = null;
			}
		} catch (CoreException e) {
			project = null;
		}

		try {
			window.getActivePage().openEditor(
					new ECLiPSeExecutionEnvironmentEditorInput(project, false,
							true), ECLiPSeExecutionEnvironment.ID);
		} catch (PartInitException e) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught whilst launching execution environment: ",
					e);
			ErrorDialog.openError(window.getShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}

	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	ISelection selection = null;
}