/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.launchers;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.LaunchConfigurationDelegate;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.runtime.Environment;

public class SarosConsoleLaunchConfigurationDelegate extends
		LaunchConfigurationDelegate {
	/*
	 * implements ILaunchConfigurationDelegate {
	 */

	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {
		Environment environment = null;
		try {
			environment = new Environment(
					Environment.PROJECT_LAUNCH_PROPERTIES,
					(IProject) (configuration.getMappedResources()[0]), null);
			boolean hasLaunchQuery = configuration.getAttribute(
					PreferenceConstants.HasLaunchQuery, false);
			String launchFile = configuration.getAttribute(
					PreferenceConstants.LaunchFile, "");
			String launchModule = configuration.getAttribute(
					PreferenceConstants.LaunchModule, "");
			String launchQuery = configuration.getAttribute(
					PreferenceConstants.LaunchQuery, "");
			String runMethod = configuration.getAttribute(
					PreferenceConstants.LaunchRunMode, "");
			environment.setLaunchQuery(hasLaunchQuery, launchFile,
					launchModule, runMethod, launchQuery);
		} catch (Exception e) {
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught whilst launching execution environment: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
		if (environment != null) {
			new ECLiPSeConsole(true, environment);
		}
		monitor.done();
	}
}
