/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.launchers;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.LaunchConfigurationDelegate;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironmentEditorInput;

public class SarosLaunchConfigurationDelegate extends
		LaunchConfigurationDelegate {

	public void launch(final ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {

		Runnable lr = new Runnable() {
			public void run() {
				try {
					IWorkbenchPage page = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();

					IProject project = (IProject) configuration
							.getMappedResources()[0];

					if (page == null || project == null) {
						return;
					}

					boolean hasLaunchQuery = configuration.getAttribute(
							PreferenceConstants.HasLaunchQuery, false);
					String launchFile = configuration.getAttribute(
							PreferenceConstants.LaunchFile, "");
					String launchModule = configuration.getAttribute(
							PreferenceConstants.LaunchModule, "");
					String launchQuery = configuration.getAttribute(
							PreferenceConstants.LaunchQuery, "");
					String runMethod = configuration.getAttribute(
							PreferenceConstants.LaunchRunMode, "");

					ECLiPSeExecutionEnvironmentEditorInput editorInput = new ECLiPSeExecutionEnvironmentEditorInput(
							project, true);
					editorInput.setLaunchQuery(hasLaunchQuery, launchFile,
							launchModule, runMethod, launchQuery);

					page
							.openEditor(editorInput,
									ECLiPSeExecutionEnvironment.ID);
				} catch (Exception e) {
					Status status = new Status(
							IStatus.ERROR,
							"Saros",
							0,
							"Exception caught whilst launching execution environment: ",
							e);
					ErrorDialog.openError(Saros.getDefault().getWorkbench()
							.getDisplay().getActiveShell(),
							"Saros Development Plugin Error", e.getMessage(),
							status);
				}
			}
		};
		Display.getDefault().syncExec(lr);
		monitor.done();
	}
}
