package org.eclipseclp.saros.development.launchers;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipseclp.saros.core.preferences.LaunchQueryComposite;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.builders.SarosNature;
import org.eclipseclp.saros.development.editors.ECLiPSeSourceEditor;

public class SarosLaunchTab extends AbstractLaunchConfigurationTab {

	public void createControl(Composite parent) {
		launchQueryGroup = new LaunchQueryComposite(parent, SWT.NONE, false);
		launchQueryGroup.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setDirty(true);
				updateLaunchConfigurationDialog();
			}
		}, new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				setDirty(true);
				updateLaunchConfigurationDialog();
			}
		});
		setControl(launchQueryGroup);
	}

	public String getName() {
		return PreferenceMessages.getString("ECLiPSeStrings.Query");
	}

	public void initializeFrom(ILaunchConfiguration configuration) {
		try {
			if (configuration.getMappedResources() != null) {
				project = (IProject) (configuration.getMappedResources())[0];
			}
			if (project != null) {
				launchQueryGroup.setProject(project);
				boolean hasLaunchQuery = configuration.getAttribute(
						PreferenceConstants.HasLaunchQuery, false);
				String launchFile = configuration.getAttribute(
						PreferenceConstants.LaunchFile, "");
				String launchModule = configuration.getAttribute(
						PreferenceConstants.LaunchModule, "");
				String launchQuery = configuration.getAttribute(
						PreferenceConstants.LaunchQuery, "");
				String runMethod = configuration.getAttribute(
						PreferenceConstants.LaunchRunMode, "");
				launchQueryGroup.setWidgetValues(hasLaunchQuery, launchFile,
						launchModule, runMethod, launchQuery);
				setMessage(PreferenceMessages
						.getString("ECLiPSeStrings.CreateProjectLaunchConfiguration")
						+ " " + project.getName() + ":");
				updateLaunchConfigurationDialog();
				return;
			} else {
				project = null;
			}
		} catch (CoreException e) {
			project = null;
		}

		initializeFromActiveSelection();
	}

	private void initializeFromActiveSelection() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();

		ISelection selection = page.getSelection();
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				if (obj instanceof IFile) {
					project = ((IFile) obj).getProject();
				} else if (obj instanceof IProject) {
					project = (IProject) obj;
				}
			}
		}

		if (project == null) {
			IEditorPart part = page.getActiveEditor();
			if ((part instanceof ECLiPSeSourceEditor)) {
				project = ((FileEditorInput) ((ECLiPSeSourceEditor) part)
						.getEditorInput()).getFile().getProject();
			}
		}

		try {
			if (project == null
					|| project.getNature(SarosNature.NATURE_ID) == null) {
				setErrorMessage(PreferenceMessages
						.getString("ECLiPSeStrings.NoActiveProject")
						+ ".");
				updateLaunchConfigurationDialog();
				return;
			}
		} catch (CoreException e) {
			project = null;
			setErrorMessage(PreferenceMessages
					.getString("ECLiPSeStrings.NoActiveProject")
					+ ".");
			updateLaunchConfigurationDialog();
			Status status = new Status(
					IStatus.ERROR,
					"Saros",
					0,
					"Exception caught whilst launching an ECLiPSe execution environment: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		launchQueryGroup.setProject(project);
		launchQueryGroup.revertToDefaults();
		launchQueryGroup.setEnabled(true);
		setMessage(PreferenceMessages
				.getString("ECLiPSeStrings.CreateProjectLaunchConfiguration")
				+ " " + project.getName() + ":");
		updateLaunchConfigurationDialog();
	}

	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		launchQueryGroup.commit();
		configuration.setMappedResources(new IResource[] { project });
		configuration.setAttribute(PreferenceConstants.HasLaunchQuery,
				launchQueryGroup.hasLaunchQuery());
		configuration.setAttribute(PreferenceConstants.LaunchFile,
				launchQueryGroup.getFileName());
		configuration.setAttribute(PreferenceConstants.LaunchModule,
				launchQueryGroup.getModuleName());
		configuration.setAttribute(PreferenceConstants.LaunchQuery,
				launchQueryGroup.getQuery());
		configuration.setAttribute(PreferenceConstants.LaunchRunMode,
				launchQueryGroup.getRunMethod());
	}

	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
	}

	private LaunchQueryComposite launchQueryGroup = null;

	private IProject project = null;
}
