/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.perspectives;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

public class SarosExecution implements IPerspectiveFactory {

	public void createInitialLayout(IPageLayout layout) {
		initialiseActions(layout);
		initialiseLayout(layout);
	}

	private void initialiseActions(IPageLayout layout) {
		layout.addShowViewShortcut(IPageLayout.ID_RES_NAV);
		layout.addShowViewShortcut(IPageLayout.ID_BOOKMARKS);
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.Projects");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.CompileScratchpad");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.DelayedGoals");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.PredicateBrowser");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.Inspector");
		// layout.addShowViewShortcut("org.eclipseclp.saros.development.views.SourceFileManager");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.StatisticsSummary");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.CallStack");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.TraceLog");
		layout
				.addShowViewShortcut("org.eclipseclp.saros.development.views.SarosConsole");
		layout
				.addPerspectiveShortcut("org.eclipseclp.saros.development.perspectives.SarosDevelopment");
	}

	private void initialiseLayout(IPageLayout layout) {
		// Get the editor area.
		String editorArea = layout.getEditorArea();

		// Top left: Resource Navigator view and Bookmarks view placeholder
		IFolderLayout topLeft = layout.createFolder("topLeft",
				IPageLayout.LEFT, 0.20f, editorArea);
		topLeft.addView("org.eclipseclp.saros.development.views.Projects");
		topLeft.addView(IPageLayout.ID_RES_NAV);
		topLeft.addPlaceholder(IPageLayout.ID_BOOKMARKS);

		// Top right:
		IFolderLayout topRight = layout.createFolder("topRight",
				IPageLayout.RIGHT, 0.75f, editorArea);
		topRight
				.addView("org.eclipseclp.saros.development.views.StatisticsSummary");
		// topRight.addView("org.eclipseclp.saros.development.views.SourceFileManager");

		// Bottom right: Console
		IFolderLayout bottomRight = layout.createFolder("bottomRight",
				IPageLayout.BOTTOM, 0.66f, editorArea);
		bottomRight
				.addView("org.eclipseclp.saros.development.views.CompileScratchpad");
		bottomRight.addView("org.eclipseclp.saros.development.views.CallStack");
		bottomRight.addView("org.eclipseclp.saros.development.views.TraceLog");
		bottomRight
				.addView("org.eclipseclp.saros.development.views.PredicateBrowser");
		bottomRight.addView("org.eclipseclp.saros.development.views.Inspector");
		bottomRight
				.addView("org.eclipseclp.saros.development.views.DelayedGoals");
		bottomRight
				.addView("org.eclipseclp.saros.development.views.SarosConsole");
	}
}
