/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipseclp.saros.core.Saros;
import org.eclipseclp.saros.core.preferences.CompilerOutput;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;

public class CompilerPage extends PropertyPage implements SelectionListener {

	public CompilerPage() {
		super();
	}

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		project = ((IProject) getElement());

		setDescription(PreferenceMessages.getString("ECLiPSeStrings.Compiler")
				+ ":");

		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData gridData = new GridData(GridData.FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		composite.setLayoutData(gridData);
		projectSpecificCheckbox = new Button(composite, SWT.CHECK);
		projectSpecificCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectSpecificSettings"));
		projectSpecificCheckbox.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectSpecificCheckbox.setLayoutData(gridData);
		configurePreferencesLink = new Link(composite, SWT.NONE);
		configurePreferencesLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureWorkspaceSettings")
				+ "...</A>");
		configurePreferencesLink.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		configurePreferencesLink.setLayoutData(gridData);
		Label separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);
		compiler = new CompilerOutput(composite, SWT.NONE, false);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		compiler.setLayoutData(gridData);
		setWidgetValues();
		return composite;
	}

	private void setWidgetValues() {
		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.ECLiPSeFcompileTarget)).equals(
					PreferenceConstants.True)) {
				compiler.setFcompile(true);
			} else {
				compiler.setFcompile(false);
			}
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.ECLiPSeIcompileTarget)).equals(
					PreferenceConstants.True)) {
				compiler.setIcompile(true);
			} else {
				compiler.setIcompile(false);
			}

			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.ECLiPSeEciToHtml)).equals(
					PreferenceConstants.True)) {
				compiler.setEciToHtml(true);
			} else {
				compiler.setEciToHtml(false);
			}
			if (project
					.getPersistentProperty(
							new QualifiedName(
									PreferenceConstants.SarosPlugin,
									PreferenceConstants.IsProjectSpecificCompilationTarget))
					.equals(PreferenceConstants.True)) {
				projectSpecificCheckbox.setSelection(true);
				configurePreferencesLink.setEnabled(false);
				compiler.setEnabled(true);
			} else {
				projectSpecificCheckbox.setSelection(false);
				configurePreferencesLink.setEnabled(true);
				compiler.setEnabled(false);
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving project properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void performDefaults() {
		compiler.revertToDefaults();
		projectSpecificCheckbox.setSelection(false);
		configurePreferencesLink.setEnabled(true);
		compiler.setEnabled(false);
		super.performDefaults();
	}

	public boolean performOk() {
		try {
			project.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.IsProjectSpecificCompilationTarget), ""
					+ projectSpecificCheckbox.getSelection());
			if (projectSpecificCheckbox.getSelection()) {
				compiler.commit();
				boolean[] outputTargets = compiler
						.getOutputCompilationTargets();
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.ECLiPSeFcompileTarget), ""
						+ outputTargets[0]);
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.ECLiPSeIcompileTarget), ""
						+ outputTargets[1]);
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.ECLiPSeEciToHtml), ""
						+ outputTargets[2]);
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst setting project properties: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		return true;
	}

	public void widgetDefaultSelected(SelectionEvent event) {

	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == projectSpecificCheckbox) {
			if (projectSpecificCheckbox.getSelection()) {
				compiler.setEnabled(true);
				configurePreferencesLink.setEnabled(false);
			} else {
				compiler.setEnabled(false);
				configurePreferencesLink.setEnabled(true);
			}
		} else if (event.getSource() == configurePreferencesLink) {
			compiler.configureWorkspacePreferences();
			// wigets will get updated, some maybe re-enabled
			if (!projectSpecificCheckbox.getSelection()) {
				compiler.setEnabled(false);
			}
		}
	}

	private CompilerOutput compiler = null;

	private Button projectSpecificCheckbox = null;

	private Link configurePreferencesLink = null;

	private IProject project = null;
}
