/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.core.preferences.EngineInstallationSelector;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;

public class ECLiPSeEngineInstallationPage extends PropertyPage implements
		SelectionListener {

	public ECLiPSeEngineInstallationPage() {
		super();
	}

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		project = (IProject) getElement();
		setDescription(PreferenceMessages
				.getString("ECLiPSeStrings.ECLiPSeEngineInstallation")
				+ ":");

		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData gridData = new GridData(GridData.FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		composite.setLayoutData(gridData);
		projectSpecificCheckbox = new Button(composite, SWT.CHECK);
		projectSpecificCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectSpecificSettings"));
		projectSpecificCheckbox.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectSpecificCheckbox.setLayoutData(gridData);
		configurePreferencesLink = new Link(composite, SWT.NONE);
		configurePreferencesLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureWorkspaceSettings")
				+ "...</A>");
		configurePreferencesLink.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		configurePreferencesLink.setLayoutData(gridData);
		Label separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);

		engineInstallation = new EngineInstallationSelector(composite,
				SWT.NONE, false);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		engineInstallation.setLayoutData(gridData);
		setWidgetValues();
		return composite;
	}

	private void setWidgetValues() {
		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.IsProjectSpecificInstallation))
					.equals(PreferenceConstants.True)) {
				projectSpecificCheckbox.setSelection(true);
				String installation = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeInstallationName));
				String installations[] = engineInstallation
						.getInstallationNames();
				for (int count = 0; count < installations.length; count++) {
					if (installations[count].equals(installation)) {
						engineInstallation
								.setSpecificInstallation(installation);
						break;
					}
				}
			} else {
				engineInstallation.setEnabled(false);
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving project properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
	}

	protected void performDefaults() {
		engineInstallation.revertToDefaults();
		engineInstallation.setEnabled(false);
		projectSpecificCheckbox.setSelection(false);
		configurePreferencesLink.setEnabled(true);
		super.performDefaults();
	}

	public boolean performOk() {
		try {
			project.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.IsProjectSpecificInstallation), ""
					+ projectSpecificCheckbox.getSelection());
			if (projectSpecificCheckbox.getSelection()) {
				engineInstallation.commit();
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.IsProjectSpecificInstallation), ""
						+ engineInstallation.isProjectSpecificInstallation());
				if (engineInstallation.isProjectSpecificInstallation()) {
					project.setPersistentProperty(new QualifiedName(
							PreferenceConstants.SarosPlugin,
							PreferenceConstants.ECLiPSeInstallationName),
							engineInstallation.getInstallationName());
				}
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst setting project properties: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		return true;
	}

	public void widgetDefaultSelected(SelectionEvent event) {

	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == projectSpecificCheckbox) {
			if (projectSpecificCheckbox.getSelection()) {
				engineInstallation.setEnabled(true);
				configurePreferencesLink.setEnabled(false);
			} else {
				engineInstallation.setEnabled(false);
				configurePreferencesLink.setEnabled(true);
			}
		} else if (event.getSource() == configurePreferencesLink) {
			engineInstallation.configureWorkspacePreferences();
		}
	}

	private EngineInstallationSelector engineInstallation = null;

	private Button projectSpecificCheckbox = null;

	private Link configurePreferencesLink = null;

	private IProject project = null;
}
