/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipseclp.saros.core.preferences.ECLiPSeEngineSettings;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.Saros;

public class ECLiPSeEngineSettingsPage extends PropertyPage implements
		SelectionListener {

	public ECLiPSeEngineSettingsPage() {
		super();
	}

	protected Control createContents(Composite parent) {
		project = (IProject) getElement();

		setDescription(PreferenceMessages
				.getString("ECLiPSeStrings.EngineSettings")
				+ ":");

		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData gridData = new GridData(GridData.FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		composite.setLayoutData(gridData);
		projectSpecificCheckbox = new Button(composite, SWT.CHECK);
		projectSpecificCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectSpecificSettings"));
		projectSpecificCheckbox.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectSpecificCheckbox.setLayoutData(gridData);
		configurePreferencesLink = new Link(composite, SWT.NONE);
		configurePreferencesLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureWorkspaceSettings")
				+ "...</A>");
		configurePreferencesLink.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		configurePreferencesLink.setLayoutData(gridData);
		Label separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);

		installation = initialiseInstallation(true);
		engineSettings = new ECLiPSeEngineSettings(composite, SWT.NONE, false,
				installation);
		if (projectSpecificCheckbox.getSelection()) {
			configurePreferencesLink.setEnabled(false);
			engineSettings.setEnabled(true);
		} else {
			configurePreferencesLink.setEnabled(true);
			engineSettings.setEnabled(false);
		}
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 2;
		engineSettings.setLayoutData(gridData);
		return composite;
	}

	private EnginePreferences initialiseInstallation(boolean getProperties) {
		EnginePreferences installation = null;
		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.IsProjectSpecificInstallation))
					.equals(PreferenceConstants.True)) {
				String installationName = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeInstallationName));
				installation = PreferenceManager
						.getInstallationByName(installationName);
				if (installation == null) {
					installation = PreferenceManager.getDefaultInstallation();
				}
			} else {
				installation = PreferenceManager.getDefaultInstallation();
			}
			if (installation == null) {
				installation = PreferenceManager.getStoreDefaultInstallation();
			}
			if (getProperties
					&& project
							.getPersistentProperty(
									new QualifiedName(
											PreferenceConstants.SarosPlugin,
											PreferenceConstants.IsProjectSpecificEngineSettings))
							.equals(PreferenceConstants.True)) {
				projectSpecificCheckbox.setSelection(true);
				// Get the properties
				installation.setFlag(PreferenceConstants.afterEventTimer,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.afterEventTimer)));
				installation.setFlag(PreferenceConstants.allDynamic, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.allDynamic)));
				installation.setFlag(PreferenceConstants.brealExceptions,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.brealExceptions)));
				installation.setFlag(PreferenceConstants.coroutine, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.coroutine)));
				installation.setFlag(PreferenceConstants.debugging, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.debugging)));
				installation.setFlag(PreferenceConstants.debugCompile, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.debugCompile)));
				installation.setFlag(PreferenceConstants.enableInterrupts,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.enableInterrupts)));
				installation.setFlag(PreferenceConstants.goalExpansion, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.goalExpansion)));
				installation.setFlag(PreferenceConstants.macroExpansion,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.macroExpansion)));
				installation.setFlag(PreferenceConstants.preferRationals,
						project.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.preferRationals)));
				installation.setFlag(PreferenceConstants.variableNames, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.variableNames)));
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving project properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		return installation;
	}

	protected void performDefaults() {
		installation = initialiseInstallation(false);
		engineSettings.revertToDefaults(installation);
		engineSettings.setEnabled(false);
		configurePreferencesLink.setEnabled(true);
		projectSpecificCheckbox.setSelection(false);
		super.performDefaults();
	}

	public boolean performOk() {
		try {
			project.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.IsProjectSpecificEngineSettings), ""
					+ projectSpecificCheckbox.getSelection());
			if (projectSpecificCheckbox.getSelection()) {
				installation = engineSettings.commit(installation);
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.afterEventTimer), installation
						.getStringFlag(PreferenceConstants.afterEventTimer));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.allDynamic), installation
						.getStringFlag(PreferenceConstants.allDynamic));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.brealExceptions), installation
						.getStringFlag(PreferenceConstants.brealExceptions));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.coroutine), installation
						.getStringFlag(PreferenceConstants.coroutine));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.debugging), installation
						.getStringFlag(PreferenceConstants.debugging));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.debugCompile), installation
						.getStringFlag(PreferenceConstants.debugCompile));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.enableInterrupts), installation
						.getStringFlag(PreferenceConstants.enableInterrupts));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.goalExpansion), installation
						.getStringFlag(PreferenceConstants.goalExpansion));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.macroExpansion), installation
						.getStringFlag(PreferenceConstants.macroExpansion));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.preferRationals), installation
						.getStringFlag(PreferenceConstants.preferRationals));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.variableNames), installation
						.getStringFlag(PreferenceConstants.variableNames));
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst setting project properties: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
		return true;
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == projectSpecificCheckbox) {
			if (projectSpecificCheckbox.getSelection()) {
				engineSettings.setEnabled(true);
				configurePreferencesLink.setEnabled(false);
			} else {
				engineSettings.setEnabled(false);
				configurePreferencesLink.setEnabled(true);
			}
		} else if (event.getSource() == configurePreferencesLink) {
			PreferencesUtil
					.createPreferenceDialogOn(
							getShell(),
							PreferenceConstants.EngineInstallationPreferencePageId,
							new String[] {
									PreferenceConstants.EngineInstallationPreferencePageId,
									PreferenceConstants.EngineSettingsPreferencePageId,
									PreferenceConstants.GCPreferencePageId,
									PreferenceConstants.OutputOptionsPreferencePageId },
							null).open();
			installation = initialiseInstallation(false);
			engineSettings.setWidgetValues(installation);
		}
	}

	private Button projectSpecificCheckbox = null;

	private Link configurePreferencesLink = null;

	private ECLiPSeEngineSettings engineSettings = null;

	private IProject project = null;

	private EnginePreferences installation = null;
}
