/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipseclp.saros.core.Saros;
import org.eclipseclp.saros.core.preferences.ECLiPSeGCSettings;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;

public class ECLiPSeGCSettingsPage extends PropertyPage implements
		SelectionListener {

	public ECLiPSeGCSettingsPage() {
		super();
	}

	protected Control createContents(Composite parent) {
		project = (IProject) getElement();

		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData gridData = new GridData(GridData.FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		composite.setLayoutData(gridData);
		projectSpecificCheckbox = new Button(composite, SWT.CHECK);
		projectSpecificCheckbox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectSpecificSettings"));
		projectSpecificCheckbox.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectSpecificCheckbox.setLayoutData(gridData);
		configurePreferencesLink = new Link(composite, SWT.NONE);
		configurePreferencesLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureWorkspaceSettings")
				+ "...</A>");
		configurePreferencesLink.addSelectionListener(this);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		configurePreferencesLink.setLayoutData(gridData);
		Label separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);
		installation = initialiseInstallation(true);
		eclipseGCsettings = new ECLiPSeGCSettings(composite, SWT.NONE,
				installation);
		if (projectSpecificCheckbox.getSelection()) {
			configurePreferencesLink.setEnabled(false);
			eclipseGCsettings.setEnabled(true);
		} else {
			configurePreferencesLink.setEnabled(true);
			eclipseGCsettings.setEnabled(false);
		}
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 2;
		eclipseGCsettings.setLayoutData(gridData);
		return composite;
	}

	private EnginePreferences initialiseInstallation(boolean getProperties) {
		EnginePreferences installation = null;
		try {
			if (project.getPersistentProperty(
					new QualifiedName(PreferenceConstants.SarosPlugin,
							PreferenceConstants.IsProjectSpecificInstallation))
					.equals(PreferenceConstants.True)) {
				String installationName = project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.ECLiPSeInstallationName));
				installation = PreferenceManager
						.getInstallationByName(installationName);
				if (installation == null) {
					installation = PreferenceManager.getDefaultInstallation();
				}
			} else {
				installation = PreferenceManager.getDefaultInstallation();
			}
			if (installation == null) {
				installation = PreferenceManager.getStoreDefaultInstallation();
			}
			if (getProperties
					&& project
							.getPersistentProperty(
									new QualifiedName(
											PreferenceConstants.SarosPlugin,
											PreferenceConstants.IsProjectSpecificGCsettings))
							.equals(PreferenceConstants.True)) {
				projectSpecificCheckbox.setSelection(true);
				// Get the properties
				installation.setFlag(PreferenceConstants.gc, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.gc)));
				installation.setFlag(PreferenceConstants.gcPolicy, project
						.getPersistentProperty(new QualifiedName(
								PreferenceConstants.SarosPlugin,
								PreferenceConstants.gcPolicy)));
				installation.setFlag(PreferenceConstants.gcInterval,
						new Integer(project
								.getPersistentProperty(new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.gcInterval))));
				installation.setFlag(PreferenceConstants.gcIntervalDict,
						new Integer(project
								.getPersistentProperty(new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.gcIntervalDict))));
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst retrieving project properties: ",
					e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		return installation;
	}

	protected void performDefaults() {
		eclipseGCsettings.revertToDefaults(installation);
		eclipseGCsettings.setEnabled(false);
		configurePreferencesLink.setEnabled(true);
		projectSpecificCheckbox.setSelection(false);
		super.performDefaults();
	}

	public boolean performOk() {
		try {
			project.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.IsProjectSpecificGCsettings), ""
					+ projectSpecificCheckbox.getSelection());
			if (projectSpecificCheckbox.getSelection()) {
				installation = eclipseGCsettings.commit(installation);
				project.setPersistentProperty(
						new QualifiedName(PreferenceConstants.SarosPlugin,
								PreferenceConstants.gc), installation
								.getStringFlag(PreferenceConstants.gc));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.gcPolicy), installation
						.getStringFlag(PreferenceConstants.gcPolicy));
				project.setPersistentProperty(new QualifiedName(
						PreferenceConstants.SarosPlugin,
						PreferenceConstants.gcInterval), ""
						+ installation
								.getIntFlag(PreferenceConstants.gcInterval));
				project
						.setPersistentProperty(
								new QualifiedName(
										PreferenceConstants.SarosPlugin,
										PreferenceConstants.gcIntervalDict),
								""
										+ installation
												.getIntFlag(PreferenceConstants.gcIntervalDict));
			}
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst setting project properties: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}

		return true;
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == projectSpecificCheckbox) {
			if (projectSpecificCheckbox.getSelection()) {
				eclipseGCsettings.setEnabled(true);
				configurePreferencesLink.setEnabled(false);
			} else {
				eclipseGCsettings.setEnabled(false);
				configurePreferencesLink.setEnabled(true);
			}
		} else if (event.getSource() == configurePreferencesLink) {
			PreferencesUtil
					.createPreferenceDialogOn(
							getShell(),
							PreferenceConstants.EngineInstallationPreferencePageId,
							new String[] {
									PreferenceConstants.EngineInstallationPreferencePageId,
									PreferenceConstants.EngineSettingsPreferencePageId,
									PreferenceConstants.GCPreferencePageId,
									PreferenceConstants.OutputOptionsPreferencePageId },
							null).open();
			installation = initialiseInstallation(false);
			eclipseGCsettings.setWidgetValues(installation);
		}
	}

	private Button projectSpecificCheckbox = null;

	private Link configurePreferencesLink = null;

	private ECLiPSeGCSettings eclipseGCsettings = null;

	private IProject project = null;

	private EnginePreferences installation = null;
}
