/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.properties;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipseclp.saros.core.preferences.LaunchQueryComposite;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.Saros;

public class ProjectLaunchDefaultsPage extends PropertyPage {

	public ProjectLaunchDefaultsPage() {
		super();
	}

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		setDescription(PreferenceMessages
				.getString("ECLiPSeStrings.LaunchDefaults")
				+ ":");

		launchQuery = new LaunchQueryComposite(parent, SWT.NONE, false,
				(IResource) getElement());
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		launchQuery.setLayout(new GridLayout());
		launchQuery.setLayoutData(gridData);

		return launchQuery;
	}

	protected void performDefaults() {
		super.performDefaults();
		launchQuery.revertToDefaults();
		launchQuery.setEnabled(false);
	}

	public boolean performOk() {
		IResource resource = (IResource) getElement();
		launchQuery.commit();
		try {
			resource.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.HasLaunchQuery), ""
					+ launchQuery.hasLaunchQuery());
			resource.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.LaunchFile), launchQuery.getFileName());
			resource.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.LaunchModule), launchQuery
					.getModuleName());
			resource.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.LaunchQuery), launchQuery.getQuery());
			resource.setPersistentProperty(new QualifiedName(
					PreferenceConstants.SarosPlugin,
					PreferenceConstants.LaunchRunMode), launchQuery
					.getRunMethod());
		} catch (CoreException e) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"Exception caught whilst setting project properties: ", e);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(),
					"Saros Development Plugin Error", e.getMessage(), status);
		}
		return true;
	}

	private LaunchQueryComposite launchQuery = null;
}
