/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class CompileScratchpad extends ViewPart implements
		SarosViewNotification {

	public void createPartControl(Composite parent) {
		compileText = new StyledText(parent, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		compileAllItem = new MenuItem(popupMenu, SWT.PUSH);
		compileAllItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CompileAll"));
		compileAllItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				compile(compileText.getText());
			}
		});
		compileSelectionItem = new MenuItem(popupMenu, SWT.PUSH);
		compileSelectionItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CompileSelection"));
		compileSelectionItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				compile(compileText.getSelectionText());
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		clearItem = new MenuItem(popupMenu, SWT.PUSH);
		clearItem.setText(PreferenceMessages.getString("ECLiPSeStrings.Clear"));
		clearItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				clearText();
			}
		});
		compileText.setMenu(popupMenu);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			enableWidgets(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	private void enableWidgets(boolean enable) {
		compileAllItem.setEnabled(enable);
		compileSelectionItem.setEnabled(enable);
		clearItem.setEnabled(enable);
		compileText.setEditable(enable);
	}

	protected void compile(String text) {
		IEditorPart editor = this.getSite().getPage().getActiveEditor();
		String failMessage = "Unable to perform scratchpad compilation.";
		if (!(editor instanceof ECLiPSeExecutionEnvironment)) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"A Saros ECLiPSe Execution Environment must be active.",
					null);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Plugin Error",
					failMessage, status);
			return;
		}
		((ECLiPSeExecutionEnvironment) editor).interruptingRpc(
				new CompoundTermImpl(":", new Atom("tracer_tcl"),
						new CompoundTermImpl("compile_string", text)),
				failMessage);
	}

	protected void clearText() {
		compileText.replaceTextRange(0, compileText.getCharCount(), "");
	}

	public void setFocus() {
		compileText.setFocus();
	}

	public void configureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		if (activeInteraction != null) {
			activeInteraction
					.updateCompileScratchpadData(compileText.getText());
		}
		activeInteraction = eclipseInteraction;
		compileText.setText(eclipseInteraction.getCompileScratchpadData());
		enableWidgets(true);
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			activeInteraction
					.updateCompileScratchpadData(compileText.getText());
		}
		deconfigureEngine(eclipseInteraction);
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction) {
			activeInteraction = null;
			clearText();
			enableWidgets(false);
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
		eclipseInteraction.updateCompileScratchpadData(compileText.getText());
	}

	StyledText compileText = null;

	private Menu popupMenu = null;

	private ECLiPSeInteractionInterface activeInteraction = null;

	private MenuItem compileAllItem = null;

	private MenuItem compileSelectionItem = null;

	private MenuItem clearItem = null;
}
