/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import java.util.ArrayList;
import java.util.Collection;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.EnginePreferences;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.core.preferences.PrintDepthOutputOptionsShell;
import org.eclipseclp.saros.development.Saros;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeDelayedGoal;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class DelayedGoals extends ViewPart implements SarosViewNotification,
		MouseListener {

	public void createPartControl(Composite parent) {
		delayedGoalsText = new StyledText(parent, SWT.MULTI | SWT.READ_ONLY
				| SWT.H_SCROLL | SWT.V_SCROLL);
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		predicateInfoItem = new MenuItem(popupMenu, SWT.PUSH);
		predicateInfoItem.setEnabled(false);
		new MenuItem(popupMenu, SWT.SEPARATOR);
		refreshGoalsItem = new MenuItem(popupMenu, SWT.PUSH);
		refreshGoalsItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Refresh"));
		refreshGoalsItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				requestDelayedGoalsRefresh();
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		spyGoalItem = new MenuItem(popupMenu, SWT.CHECK);
		spyGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SpyGoal"));
		spyGoalItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				spyCurrentGoal();
			}
		});
		inspectGoalItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectGoal"));
		inspectGoalItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				// activeInteraction.inspectGoal(currentTraceLine);
			}
		});
		/*
		 * observeGoalItem = new MenuItem(popupMenu, SWT.PUSH);
		 * observeGoalItem.setText(PreferenceMessages
		 * .getString("ECLiPSeStrings.ObserveGoal"));
		 * observeGoalItem.addSelectionListener(new SelectionAdapter() { public
		 * void widgetSelected(SelectionEvent event) { //
		 * activeInteraction.observeGoal(currentTraceLine); } });
		 */
		new MenuItem(popupMenu, SWT.SEPARATOR);
		filterMenuItem = new MenuItem(popupMenu, SWT.CASCADE);
		filterMenuItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Filter"));
		Menu menu = new Menu(popupMenu);
		filterMenuItem.setMenu(menu);
		traceableGoalItem = new MenuItem(menu, SWT.CHECK);
		traceableGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.TraceableGoals"));
		traceableGoalItem.setSelection(true);
		scheduledGoalItem = new MenuItem(menu, SWT.CHECK);
		scheduledGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ScheduledGoals"));
		spiedGoalItem = new MenuItem(menu, SWT.CHECK);
		spiedGoalItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SpiedGoals"));
		suspendedTriggerItem = new MenuItem(menu, SWT.CASCADE);
		suspendedTriggerItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SuspendedOnTrigger"));
		triggerMenu = new Menu(menu);
		refreshTriggersItem = new MenuItem(triggerMenu, SWT.PUSH);
		refreshTriggersItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RefreshTriggerList"));
		refreshTriggersItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				requestTriggerListRefresh();
			}
		});
		eventRefreshItem = new MenuItem(popupMenu, SWT.CHECK);
		eventRefreshItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RefreshEachTraceEvent"));
		eventRefreshItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction
						.setAutoRefreshingDelayedGoals(eventRefreshItem
								.getSelection());
			}
		});
		new MenuItem(triggerMenu, SWT.SEPARATOR);
		suspendedTriggerItem.setMenu(triggerMenu);
		new MenuItem(popupMenu, SWT.SEPARATOR);
		printOptionsItem = new MenuItem(popupMenu, SWT.PUSH);
		printOptionsItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ChangeOutputOptions")
				+ "...");
		printOptionsItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				changeOutputOptions();
			}
		});
		delayedGoalsText.addMouseListener(this);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			enableMenuItems(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	private void buildTriggerMenu(ArrayList triggers) {
		MenuItem[] items = triggerMenu.getItems();
		for (int cnt = 2; cnt < items.length; cnt++) {
			items[cnt].dispose();
		}

		MenuItem item = null;
		for (int cnt = 0; cnt < triggers.size(); cnt++) {
			item = new MenuItem(triggerMenu, SWT.CHECK);
			item.setText(((Atom) triggers.get(cnt)).functor());
		}
	}

	private void changeOutputOptions() {
		EnginePreferences installation = (EnginePreferences) activeInteraction
				.getInstallation().clone();
		installation = (new PrintDepthOutputOptionsShell(installation))
				.open(this.getSite().getShell());

		String mode = installation
				.getStringFlag(PreferenceConstants.outputMode);
		CompoundTerm command = new CompoundTermImpl(":",
				new Atom("tracer_tcl"), new CompoundTermImpl(
						"set_tracer_output_modes", mode));
		activeInteraction.interruptingRpc(command,
				"Unable to set tracer output mode");

		Integer printDepth = new Integer(installation
				.getIntFlag(PreferenceConstants.printDepth));
		command = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("set_tracer_print_depth", printDepth));
		activeInteraction.interruptingRpc(command,
				"Unable to set tracer print depth");
	}

	private void enableMenuItems(boolean enable) {
		popupMenu.setEnabled(enable);
		refreshGoalsItem.setEnabled(enable);
		spyGoalItem.setEnabled(enable);
		inspectGoalItem.setEnabled(enable);
		// observeGoalItem.setEnabled(enable);
		filterMenuItem.setEnabled(enable);
		traceableGoalItem.setEnabled(enable);
		scheduledGoalItem.setEnabled(enable);
		spiedGoalItem.setEnabled(enable);
		suspendedTriggerItem.setEnabled(enable);
		printOptionsItem.setEnabled(enable);
		eventRefreshItem.setEnabled(enable);
		refreshTriggersItem.setEnabled(enable);
	}

	protected void clearText() {
		delayedGoalsText.replaceTextRange(0, delayedGoalsText.getCharCount(),
				"");
	}

	public void mouseDown(MouseEvent event) {
		if (event.button == 3) {
			if (popupMenu.getEnabled()) {
				// NOTE: currentIndex is currentLine - 1 - lines are prepended
				// with '\n';
				int currentIndex = delayedGoalsText.getLineIndex(event.y) - 1;
				if (currentDelayedGoals != null && currentIndex >= 0
						&& currentIndex < currentDelayedGoals.size()) {
					ECLiPSeDelayedGoal currentDelayedGoal = (ECLiPSeDelayedGoal) currentDelayedGoals
							.get(currentIndex);
					final int priority = currentDelayedGoal.getPriority();
					final int invocation = currentDelayedGoal.getInvocation();
					if (invocation != 0) {
						AsynchronousRPCresult r = new AsynchronousRPCresult() {
							public void run() {
								final CompoundTerm refreshedLine = (CompoundTerm) getResult()
										.arg(2);

								currentPredSpec = (String) refreshedLine.arg(3);
								if (currentPredSpec.equals("unknown")) {
									predicateInfoItem.setText("");
									spyGoalItem.setText("Spy Goal");
									predicateInfoItem
											.setText("No goal for invocation '"
													+ invocation
													+ "'. Please refresh.");
									spyGoalItem.setEnabled(false);
									// observeGoalItem.setEnabled(false);
									inspectGoalItem.setEnabled(false);
									currentPredSpec = null;
									currentPredModule = null;
								} else {
									currentPredModule = ((Atom) refreshedLine
											.arg(5)).functor();
									predicateInfoItem
											.setText((String) refreshedLine
													.arg(4)
													+ " @ "
													+ currentPredModule
													+ " "
													+ invocation
													+ " "
													+ " <" + priority + ">");
									spyGoalItem.setText("Spy "
											+ currentPredSpec);
									if (refreshedLine.arg(10).equals(
											PreferenceConstants.on)) {
										spyGoalItem.setSelection(true);
									} else {
										spyGoalItem.setSelection(false);
									}
									spyGoalItem.setEnabled(true);
									// observeGoalItem.setEnabled(true);
									inspectGoalItem.setEnabled(true);
								}

							}
						};
						Object args[] = {
								new Integer(currentDelayedGoal.getInvocation()),
								new Integer(0), null, null, null, null, null,
								null, null, null };
						activeInteraction.interruptingRpc(new CompoundTermImpl(
								":", new Atom("tracer_tcl"),
								new CompoundTermImpl(
										"saros_get_goal_info_by_invoc", args)),
								"Unable to obtain call stack goal info for '"
										+ currentDelayedGoal.getLineText()
										+ "'.", r, null);
					} else {
						predicateInfoItem.setText("No goal info: use tracer.");
						spyGoalItem.setEnabled(false);
						// observeGoalItem.setEnabled(false);
						inspectGoalItem.setEnabled(false);
					}
				} else {
					predicateInfoItem.setText("No goal info.");
					currentDelayedGoals = null;
					currentPredSpec = null;
					currentPredModule = null;
					spyGoalItem.setEnabled(false);
					// observeGoalItem.setEnabled(false);
					inspectGoalItem.setEnabled(false);
				}
			}
			popupMenu.setVisible(true);
		}
	}

	private void spyCurrentGoal() {
		if (currentPredSpec != null) {
			Atom spyValue;
			if (spyGoalItem.getSelection()) {
				spyValue = new Atom(PreferenceConstants.on);
			} else {
				spyValue = new Atom(PreferenceConstants.off);
			}
			CompoundTerm term = new CompoundTermImpl(":",
					new Atom("tracer_tcl"), new CompoundTermImpl(
							"set_flag_string", currentPredSpec,
							new Atom("spy"), spyValue, new Atom(
									currentPredModule)));
			activeInteraction.interruptingRpc(term,
					"Unable to set spy predicate property for '"
							+ currentPredSpec + "'.");
		}
	}

	public void mouseDoubleClick(MouseEvent event) {
	}

	public void mouseUp(MouseEvent event) {
	}

	protected void requestTriggerListRefresh() {
		CompoundTerm command = new CompoundTermImpl(":",
				new Atom("tracer_tcl"), new CompoundTermImpl("get_triggers",
						(Object) null));
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				refreshTriggerList(activeInteraction, getResult());
			}
		};
		activeInteraction.interruptingRpc(command,
				"Unable to refresh current trigger list", r, null);
	}

	protected void refreshTriggerList(
			ECLiPSeInteractionInterface eclipseInteraction, CompoundTerm result) {
		if (activeInteraction == eclipseInteraction) {
			ArrayList triggers = new ArrayList(
					(Collection) ((CompoundTerm) result.arg(2)).arg(1));
			buildTriggerMenu(triggers);
		}
	}

	private void requestDelayedGoalsRefresh() {
		IEditorPart editor = this.getSite().getPage().getActiveEditor();
		if (!(editor instanceof ECLiPSeExecutionEnvironment)) {
			Status status = new Status(IStatus.ERROR, "Saros", 0,
					"A Saros ECLiPSe Execution Environment must be active.",
					null);
			ErrorDialog.openError(Saros.getDefault().getWorkbench()
					.getDisplay().getActiveShell(), "Saros Plugin Error",
					"Unable to refresh delayed goals.", status);
			return;
		}

		refreshDelayedGoals((ECLiPSeExecutionEnvironment) editor);
	}

	public void refreshDelayedGoals(
			ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction != eclipseInteraction) {
			return;
		}
		ArrayList triggers = new ArrayList();
		MenuItem[] items = triggerMenu.getItems();
		for (int cnt = 2; cnt < items.length; cnt++) {
			if (items[cnt].getSelection()) {
				triggers.add(new Atom(items[cnt].getText()));
			}
		}

		Integer which;
		if (triggers.size() == 0) {
			which = new Integer(0);
		} else {
			which = new Integer(1);
		}

		CompoundTerm filter = new CompoundTermImpl("dg_filter",
				integerFromBoolean(traceableGoalItem.getSelection()),
				integerFromBoolean(spiedGoalItem.getSelection()),
				integerFromBoolean(scheduledGoalItem.getSelection()));

		eclipseInteraction.interruptingRpc(new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new CompoundTermImpl("gui_dg", which, triggers,
				filter)), "Unable to refresh delayed goals");
	}

	private Integer integerFromBoolean(boolean value) {
		if (value) {
			return new Integer(1);
		}

		return new Integer(0);
	}

	public void setFocus() {
		delayedGoalsText.setFocus();
	}

	public void updateDelayedGoalsData(
			ECLiPSeInteractionInterface eclipseInteraction,
			ArrayList delayedGoals) {
		if (eclipseInteraction == activeInteraction) {
			clearText();
			currentDelayedGoals = delayedGoals;
			if (currentDelayedGoals.size() == 0) {
				currentDelayedGoals = null;
			}
			currentPredSpec = null;
			currentPredModule = null;
			ECLiPSeDelayedGoal delayedGoal;
			for (int count = 0; count < delayedGoals.size(); count++) {
				delayedGoal = (ECLiPSeDelayedGoal) delayedGoals.get(count);
				delayedGoalsText.append(delayedGoal.getLineText());
			}
		}
	}

	public void configureEngine(
			final ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			requestTriggerListRefresh();
			refreshDelayedGoals(eclipseInteraction);
		}
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		activeInteraction = eclipseInteraction;
		eventRefreshItem.setSelection(eclipseInteraction
				.isAutoRefreshingDelayedGoals());
		requestTriggerListRefresh();
		refreshDelayedGoals(eclipseInteraction);
		enableMenuItems(true);
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			enableMenuItems(false);
			clearText();
			currentDelayedGoals = null;
			currentPredSpec = null;
			currentPredModule = null;
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	StyledText delayedGoalsText = null;

	private Menu popupMenu = null;

	private Menu triggerMenu = null;

	private MenuItem predicateInfoItem = null;

	private MenuItem spyGoalItem = null;

	private MenuItem inspectGoalItem = null;

	private MenuItem observeGoalItem = null;

	private MenuItem eventRefreshItem = null;

	private MenuItem refreshGoalsItem = null;

	private MenuItem filterMenuItem = null;

	private MenuItem traceableGoalItem = null;

	private MenuItem scheduledGoalItem = null;

	private MenuItem spiedGoalItem = null;

	private MenuItem suspendedTriggerItem = null;

	private MenuItem printOptionsItem = null;

	private MenuItem refreshTriggersItem = null;

	private ECLiPSeInteractionInterface activeInteraction = null;

	private ArrayList currentDelayedGoals = null;

	private String currentPredSpec = null;

	private String currentPredModule = null;
}
