/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TreeEvent;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;
import org.eclipseclp.saros.runtime.InspectorInstance;
import org.eclipseclp.saros.runtime.InspectorNode;
import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class Inspector extends ViewPart implements SarosViewNotification,
		TreeListener {

	public void createPartControl(Composite parent) {
		parentComposite = parent;
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		tabFolder = new TabFolder(parent, SWT.BORDER);
		inspectCurrentItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectCurrentItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectCurrentGoal"));
		inspectCurrentItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction.inspectCurrentGoal();
			}
		});
		inspectInvocationItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectInvocationItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectInvokedGoal")
				+ "...");
		inspectInvocationItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				activeInteraction
						.inspectInvokedGoal(getInvocationNumberToInspect());
			}
		});
		tabFolder.setMenu(popupMenu);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			enableWidgets(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	public void setFocus() {
	}

	private void enableWidgets(boolean enable) {
		tabFolder.setEnabled(enable);
		inspectCurrentItem.setEnabled(enable);
		inspectInvocationItem.setEnabled(enable);
	}

	protected int getInvocationNumberToInspect() {
		InputDialog d = new InputDialog(this.getSite().getShell(),
				"Inspect Goal Invocation", "Invocation number:", null, null);
		d.setBlockOnOpen(true);
		if (d.open() == Window.OK) {
			try {
				return new Integer(d.getValue()).intValue();
			} catch (NumberFormatException ne) {
				return -1;
			}
		} else {
			return -1;
		}
	}

	protected void setInspectorStructureDepth(InspectorInstance inspector) {
		InputDialog d = new InputDialog(this.getSite().getShell(),
				"Set Inspector Structure Depth", "Structure depth:", ""
						+ inspector.getStructureDepth(), null);
		d.setBlockOnOpen(true);
		if (d.open() == Window.OK) {
			inspector.setStructureDepth(new Integer(d.getValue()).intValue());
		} else {
			return;
		}
	}

	protected void setInspectorPrintDepth(InspectorInstance inspector) {
		InputDialog d = new InputDialog(this.getSite().getShell(),
				"Set Inspector Print Depth", "Print depth:", ""
						+ inspector.getPrintDepth(), null);
		d.setBlockOnOpen(true);
		if (d.open() == Window.OK) {
			inspector.setPrintDepth(new Integer(d.getValue()).intValue());
		} else {
			return;
		}
	}

	public void inspectCurrentGoal(
			ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction != activeInteraction) {
			return;
		}

		InspectorInstance inspector = new InspectorInstance("current", 20, 5);
		inspectors.add(inspector);
		addInspectorInstance(inspector);
	}

	public void inspectInvokedGoal(
			ECLiPSeInteractionInterface eclipseInteraction, int invocation) {
		if (eclipseInteraction != activeInteraction) {
			return;
		}

		InspectorInstance inspector = new InspectorInstance("invoc("
				+ invocation + ")", 20, 5);
		inspectors.add(inspector);
		addInspectorInstance(inspector);
	}

	private void addInspectorInstance(final InspectorInstance inspector) {
		final TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(inspector.getSource());
		Tree tree = new Tree(tabFolder, SWT.SINGLE | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER);
		Listener expansionListener = new Listener() {
			public void handleEvent(Event event) {
				TreeItem item = (TreeItem) event.item;
				InspectorNode node = (InspectorNode) item.getData();
				if (item.getExpanded()) {
					item.setText(node.getPrintTerm());
				} else {
					item.setText(node.getSummary());
				}
			}
		};
		tree.addListener(SWT.Expand, expansionListener);
		tree.addListener(SWT.Collapse, expansionListener);
		tree.addListener(SWT.DefaultSelection, new Listener() {
			public void handleEvent(Event event) {
				TreeItem item = (TreeItem) event.item;
				if (item != null) {
					if (item.getItemCount() == 0) {
						expandChildren(item);
					}
				}
			}
		});
		tabItem.setControl(tree);
		InspectorNode rootNode = inspector.getRootNode();
		Menu popupMenu = new Menu(parentComposite.getShell(), SWT.POP_UP);
		MenuItem inspectCurrentItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectCurrentItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectCurrentGoal"));
		inspectCurrentItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				inspectCurrentGoal(activeInteraction);
			}
		});
		MenuItem inspectInvocationItem = new MenuItem(popupMenu, SWT.PUSH);
		inspectInvocationItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InspectInvokedGoal")
				+ "...");
		inspectInvocationItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				inspectInvokedGoal(activeInteraction,
						getInvocationNumberToInspect());
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		MenuItem listDepthItem = new MenuItem(popupMenu, SWT.PUSH);
		listDepthItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SetStructureDepth")
				+ "...");
		listDepthItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				setInspectorStructureDepth(inspector);
			}
		});
		MenuItem printDepthItem = new MenuItem(popupMenu, SWT.PUSH);
		printDepthItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.SetPrintDepth")
				+ "...");
		printDepthItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				setInspectorPrintDepth(inspector);
			}
		});
		new MenuItem(popupMenu, SWT.SEPARATOR);
		MenuItem closeItem = new MenuItem(popupMenu, SWT.PUSH);
		closeItem.setText(PreferenceMessages.getString("ECLiPSeStrings.Close"));
		closeItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				tabItem.dispose();
				inspectors.remove(inspector);
			}
		});
		tree.setMenu(popupMenu);
		if (rootNode.getPrintTerm() == null) {
			createRoot(tree, rootNode);
		} else {
			addRoot(tree, rootNode);
		}
		tabFolder.setSelection(tabItem);
	}

	protected void expandChildren(final TreeItem parent) {
		final InspectorNode node = (InspectorNode) parent.getData();
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				LinkedList positionsList = (LinkedList) ((CompoundTerm) getResult()
						.arg(2)).arg(4);
				LinkedList childInfoList = (LinkedList) ((CompoundTerm) getResult()
						.arg(2)).arg(5);
				InspectorNode childNode;
				LinkedList childPath;
				for (int childCnt = 0; childCnt < childInfoList.size(); childCnt++) {
					childPath = (LinkedList) childInfoList.get(childCnt);
					childNode = new InspectorNode(node.getInspector(),
							(String) childPath.get(0), (String) childPath
									.get(1), ((Atom) childPath.get(2))
									.functor(), ((Integer) childPath.get(3))
									.intValue(), new ArrayList(
									(Collection) positionsList.get(childCnt)));
					addChild(parent, childNode);
				}
				parent.setText(node.getSummary());
				parent.setExpanded(true);
			}
		};
		String source = node.getInspector().getSource();
		activeInteraction.interruptingRpc(new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new CompoundTermImpl(
				"inspect_get_children_for_path", source, new CompoundTermImpl(
						"childnodes", new Atom(node.getType()), new Integer(
								node.getArity()), new Integer(node
								.getInspector().getStructureDepth()), node
								.getPath()), new Integer(node.getInspector()
						.getPrintDepth()), null, null)),
				"Unable to initiate term inspection of '" + source + "' goal.",
				r, null);
	}

	private TreeItem createRoot(Tree parent, InspectorNode node) {
		TreeItem item = new TreeItem(parent, SWT.NONE);
		createTreeItemFromPath(item, node);
		return item;
	}

	private TreeItem addRoot(Tree parent, InspectorNode node) {
		TreeItem item = new TreeItem(parent, SWT.NONE);
		createTreeItemFromNode(item, node);
		return item;
	}

	private TreeItem createChild(TreeItem parent, InspectorNode node) {
		TreeItem item = new TreeItem(parent, SWT.NONE);
		createTreeItemFromPath(item, node);
		return item;
	}

	private TreeItem addChild(TreeItem parent, InspectorNode node) {
		TreeItem item = new TreeItem(parent, SWT.NONE);
		createTreeItemFromNode(item, node);
		return item;
	}

	private void createTreeItemFromPath(final TreeItem item,
			final InspectorNode node) {
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				LinkedList info = (LinkedList) ((CompoundTerm) getResult().arg(
						2)).arg(3);
				node.setPrintTerm((String) info.get(0));
				node.setSummary((String) info.get(1));
				node.setType(((Atom) info.get(2)).functor());
				node.setArity(((Integer) info.get(3)).intValue());
				item.setText(node.getPrintTerm());
				item.setData(node);
			}
		};
		AsynchronousRPCresult s = new AsynchronousRPCresult() {
			public void run() {
				item.setText("<error inspecting subterm>");
				item.setData(node);
			}
		};
		String source = node.getInspector().getSource();
		activeInteraction.interruptingRpc(new CompoundTermImpl(":", new Atom(
				"tracer_tcl"), new CompoundTermImpl("inspect_command", source,
				new CompoundTermImpl("info", new Integer(node.getInspector()
						.getPrintDepth()), node.getPath()), null)),
				"Unable to initiate term inspection of '" + source + "' goal.",
				r, s);
	}

	private void createTreeItemFromNode(TreeItem item, InspectorNode node) {
		item.setText(node.getPrintTerm());
		item.setData(node);
		ArrayList children = node.getChildren();
		for (int cnt = 0; cnt < children.size(); cnt++) {
			addChild(item, (InspectorNode) children.get(cnt));
		}
	}

	public void configureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction) {
			inspectors.clear();
			deconfigureEngine(eclipseInteraction);
			editorActivated(eclipseInteraction);
		}
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		if (activeInteraction != null) {
			deconfigureEngine(activeInteraction);
		}
		activeInteraction = eclipseInteraction;
		inspectors = activeInteraction.getInspectorData();
		for (int cnt = 0; cnt < inspectors.size(); cnt++) {
			addInspectorInstance((InspectorInstance) inspectors.get(cnt));
		}
		enableWidgets(true);
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		deconfigureEngine(eclipseInteraction);
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			enableWidgets(false);
			activeInteraction = null;
			inspectors = null;
			TabItem[] items = tabFolder.getItems();
			for (int cnt = 0; cnt < items.length; cnt++) {
				items[cnt].dispose();
			}
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	private Menu popupMenu = null;

	private MenuItem inspectCurrentItem = null;

	private MenuItem inspectInvocationItem = null;

	private TabFolder tabFolder = null;

	private ECLiPSeInteractionInterface activeInteraction = null;

	private Composite parentComposite = null;

	private ArrayList inspectors = null;

	public void treeCollapsed(TreeEvent e) {
		// TODO Auto-generated method stub

	}

	public void treeExpanded(TreeEvent e) {
		// TODO Auto-generated method stub

	}
}
