/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.AsynchronousRPCresult;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;

public class PredicateBrowser extends ViewPart implements SarosViewNotification {

	public void createPartControl(Composite parent) {
		buttons = new ArrayList();
		buttonsEnablementState = new ArrayList();
		labels = new ArrayList();
		labelButtonHash = new Hashtable();

		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		viewScrolledComposite = new ScrolledComposite(parent, SWT.V_SCROLL
				| SWT.H_SCROLL);
		viewScrolledComposite.setLayout(new GridLayout());
		viewScrolledComposite.setLayoutData(gridData);
		viewScrolledComposite.setExpandHorizontal(true);
		viewScrolledComposite.setExpandVertical(true);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		viewComposite = new Composite(viewScrolledComposite, SWT.NONE);
		viewComposite.setLayout(new GridLayout());
		viewComposite.setLayoutData(gridData);
		viewScrolledComposite.setContent(viewComposite);
		popupMenu = new Menu(parent.getShell(), SWT.POP_UP);
		refreshViewItem = new MenuItem(popupMenu, SWT.PUSH);
		refreshViewItem.setText(PreferenceMessages
				.getString("ECLiPSeStrings.RefreshView"));
		refreshViewItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				requestViewRefresh();
			}
		});
		createModuleGroup();
		createPredicateGroup();
		createPropertiesGroup();
		viewComposite.setMenu(popupMenu);
		Point size = viewComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		viewComposite.setSize(size);
		viewScrolledComposite.setMinSize(size);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			enableWidgets(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	private void createModuleGroup() {
		moduleGroup = new Group(viewComposite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 6;
		moduleGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		moduleGroup.setLayoutData(gridData);
		moduleGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Module")); //$NON-NLS-1$
		moduleGroup.setMenu(popupMenu);
		predLabel = new Label(moduleGroup, SWT.NONE);
		predLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Predicates")); //$NON-NLS-1$
		predLabel.setMenu(popupMenu);
		predicateCombo = new Combo(moduleGroup, SWT.DROP_DOWN | SWT.READ_ONLY);
		predicateCombo.setItems(new String[] { PreferenceConstants.defined,
				PreferenceConstants.exported, PreferenceConstants.imported,
				PreferenceConstants.local, PreferenceConstants.visible }); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		predicateCombo.select(4);
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				requestRefreshPredicateList();
			}
		};
		predicateCombo.addSelectionListener(selectionAdapter);
		gridData = new GridData();
		predicateCombo.setLayoutData(gridData);
		predicateCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		moduleLabel = new Label(moduleGroup, SWT.NONE);
		moduleLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.InModule") + ":"); //$NON-NLS-1$ //$NON-NLS-2$
		moduleLabel.setMenu(popupMenu);
		moduleCombo = new Combo(moduleGroup, SWT.DROP_DOWN | SWT.READ_ONLY);
		moduleCombo.addSelectionListener(selectionAdapter);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		moduleCombo.setLayoutData(gridData);
		moduleCombo.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WIDGET_BACKGROUND));
		filterCheckBox = new Button(moduleGroup, SWT.CHECK);
		filterCheckBox.setSelection(true);
		filterCheckBox.addSelectionListener(selectionAdapter);
		filterCheckBox.setText(PreferenceMessages
				.getString("ECLiPSeStrings.FilterAuxiliary")); //$NON-NLS-1$
	}

	private void createPredicateGroup() {
		predGroup = new Group(viewComposite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		predGroup.setLayout(layout);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		predGroup.setLayoutData(gridData);
		predGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Predicates")); //$NON-NLS-1$
		predGroup.setMenu(popupMenu);
		predicateList = new List(predGroup, SWT.SINGLE | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER);
		predicateList.setMenu(popupMenu);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		predicateList.setLayoutData(gridData);
		predicateList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				displayPredicateProperties();
			}
		});
	}

	private void createPropertiesGroup() {
		propertiesGroup = new Group(predGroup, SWT.NONE);
		GridLayout layout = new GridLayout();
		propertiesGroup.setLayout(layout);
		propertiesGroup.setMenu(popupMenu);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		propertiesGroup.setLayoutData(gridData);
		propertiesGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.PredicateProperties")); //$NON-NLS-1$
		predicateLabel = new Label(propertiesGroup, SWT.BORDER | SWT.CENTER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		predicateLabel.setLayoutData(gridData);
		createRadioEntries();
	}

	private void createRadioEntries() {
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		radioEntriesComposite = new Composite(propertiesGroup, SWT.NONE);
		radioEntriesComposite.setLayout(new GridLayout());
		radioEntriesComposite.setLayoutData(gridData);
		radioEntriesComposite.setMenu(popupMenu);
		Composite composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.auxiliary,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.defined,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.debugged,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.stability,
				PreferenceConstants.statc, PreferenceConstants.dynamic, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.callType,
				PreferenceConstants.prolog, PreferenceConstants.external, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.type,
				PreferenceConstants.builtIn, PreferenceConstants.user, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.tool,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.demon,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.parallel,
				PreferenceConstants.off, PreferenceConstants.on, false);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.leash,
				PreferenceConstants.stop, PreferenceConstants.notrace, true);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.skip,
				PreferenceConstants.off, PreferenceConstants.on, true);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.startTracing,
				PreferenceConstants.off, PreferenceConstants.on, true);
		composite = new Composite(radioEntriesComposite, SWT.NONE);
		formatRadioComposite(composite, PreferenceConstants.spy,
				PreferenceConstants.off, PreferenceConstants.on, true);
		Point size = radioEntriesComposite
				.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		radioEntriesComposite.setSize(size);
	}

	/**
	 * This method initializes the radio composites
	 */
	private void formatRadioComposite(Composite composite,
			final String property, String text1, String text2, boolean enabled) {
		Button[] radioButton = new Button[2];
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		composite.setLayoutData(gridData);
		composite.setLayout(layout);
		composite.setMenu(popupMenu);
		Label propertyLabel = new Label(composite, SWT.NONE);
		labels.add(propertyLabel);
		propertyLabel.setMenu(popupMenu);
		propertyLabel.setText(longestLabel);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = propertyLabel
				.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
		gridData.minimumWidth = propertyLabel.computeSize(SWT.DEFAULT,
				SWT.DEFAULT).x;
		gridData.grabExcessHorizontalSpace = true;
		propertyLabel.setLayoutData(gridData);
		propertyLabel.setText(property + ":");
		radioButton[0] = new Button(composite, SWT.RADIO);
		radioButton[0].setText(longestOption);
		int widthHint = radioButton[0].computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
		int minWidth = radioButton[0].computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = widthHint;
		gridData.minimumWidth = minWidth;
		gridData.grabExcessHorizontalSpace = true;
		radioButton[0].setLayoutData(gridData);
		radioButton[0].setText(text1);
		radioButton[0].setMenu(popupMenu);
		radioButton[1] = new Button(composite, SWT.RADIO);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = widthHint;
		gridData.minimumWidth = minWidth;
		gridData.grabExcessHorizontalSpace = true;
		radioButton[1].setLayoutData(gridData);
		radioButton[1].setText(text2);
		buttons.add(radioButton[0]);
		buttons.add(radioButton[1]);
		buttonsEnablementState.add(new Boolean(enabled));
		buttonsEnablementState.add(new Boolean(enabled)); // ToDo Hideous hack
		// for the moment!
		labelButtonHash.put(property, radioButton);
		if (enabled) {
			SelectionAdapter adapter = new SelectionAdapter() {
				public void widgetSelected(SelectionEvent event) {
					Button button = ((Button) event.getSource());
					if (button.getSelection()) {
						setPredicateProperty(property, button.getText());
					}
				}
			};
			radioButton[0].addSelectionListener(adapter);
			radioButton[1].addSelectionListener(adapter);
		} else {
			radioButton[0].setEnabled(false);
			radioButton[1].setEnabled(false);
		}
	}

	protected void setPredicateProperty(String flag, String value) {
		CompoundTerm term = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("set_flag_string", predicateList
						.getSelection()[0], new Atom(flag), new Atom(value),
						new Atom(moduleCombo.getText())));
		activeInteraction.interruptingRpc(term,
				"Unable to set predicate property");
	}

	private void enableWidgets(boolean enable) {
		moduleGroup.setEnabled(enable);
		predGroup.setEnabled(enable);
		propertiesGroup.setEnabled(enable);
		predicateCombo.setEnabled(enable);
		moduleCombo.setEnabled(enable);
		filterCheckBox.setEnabled(enable);
		predicateList.setEnabled(enable);
		predicateLabel.setEnabled(enable);
		predLabel.setEnabled(enable);
		moduleLabel.setEnabled(enable);
		refreshViewItem.setEnabled(enable);
		for (int cnt = 0; cnt < buttons.size(); cnt++) {
			Button button = (Button) buttons.get(cnt);
			if (((Boolean) buttonsEnablementState.get(cnt)).booleanValue()) {
				button.setEnabled(enable);
			}
			if (!enable) {
				button.setSelection(false);
			}
		}
		for (int cnt = 0; cnt < labels.size(); cnt++) {
			((Label) labels.get(cnt)).setEnabled(enable);
		}
		if (!enable) {
			predicateLabel.setText("");
			predicateList.removeAll();
			moduleCombo.removeAll();
		}
	}

	protected void displayPredicateProperties() {
		if (predicateList.getSelectionCount() != 1) {
			return;
		}
		CompoundTerm term = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("flag_value",
						predicateList.getSelection()[0], new Atom(
								PreferenceConstants.definitionModule),
						new Atom(moduleCombo.getText()), (Object) null));
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				predicateLabel.setText((String) ((CompoundTerm) getResult()
						.arg(2)).arg(4));
			}
		};
		activeInteraction.interruptingRpc(term,
				"Unable to display predicate flag value", r, null);
		term = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("flag_value",
						predicateList.getSelection()[0], new Atom(
								PreferenceConstants.mode), new Atom(moduleCombo
								.getText()), (Object) null));
		r = new AsynchronousRPCresult() {
			public void run() {
				predicateLabel.setText(predicateLabel.getText() + ":"
						+ (String) ((CompoundTerm) getResult().arg(2)).arg(4));
			}
		};
		activeInteraction.interruptingRpc(term,
				"Unable to display predicate flag value", r, null);
		for (int cnt = 0; cnt < labels.size(); cnt++) {
			String labelText = ((Label) labels.get(cnt)).getText();
			// lose the ':'
			final String flag = labelText.substring(0, labelText.length() - 1);
			term = new CompoundTermImpl(":", new Atom("tracer_tcl"),
					new CompoundTermImpl("flag_value", predicateList
							.getSelection()[0], new Atom(flag), new Atom(
							moduleCombo.getText()), (Object) null));
			r = new AsynchronousRPCresult() {
				public void run() {
					String value = (String) ((CompoundTerm) getResult().arg(2))
							.arg(4);
					Button[] radioButtons = (Button[]) labelButtonHash
							.get(flag);
					for (int button = 0; button < radioButtons.length; button++) {
						if (radioButtons[button].getText().equals(value)) {
							radioButtons[button].setSelection(true);
						} else {
							radioButtons[button].setSelection(false);
						}
					}
				}
			};
			activeInteraction.interruptingRpc(term,
					"Unable to display predicate flag value", r, null);
		}
	}

	protected void requestViewRefresh() {
		enableWidgets(false);
		CompoundTerm term = new CompoundTermImpl(":", new Atom("tracer_tcl"),
				new CompoundTermImpl("list_modules", null, null));
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				refreshModuleList(activeInteraction, getResult());
				predicateCombo.select(4);
				String toplevelModule = ((Atom) ((CompoundTerm) getResult()
						.arg(2)).arg(2)).functor();
				moduleCombo.setText(toplevelModule);
				requestRefreshPredicateList();
			}
		};
		activeInteraction.interruptingRpc(term,
				"Unable to refresh current module list", r, null);
		enableWidgets(true);
	}

	protected void requestRefreshPredicateList() {
		String module = moduleCombo.getText();
		if (module.equals("")) {
			return;
		}
		CompoundTerm term = new CompoundTermImpl(
				":",
				new Atom("tracer_tcl"),
				new CompoundTermImpl("list_predicates", new Atom(predicateCombo
						.getText()), new Atom(module),
						integerFromBoolean(filterCheckBox.getSelection()), null));
		AsynchronousRPCresult r = new AsynchronousRPCresult() {
			public void run() {
				refreshPredicateList(activeInteraction, getResult());
			}
		};
		activeInteraction.interruptingRpc(term,
				"Unable to refresh current predicate list", r, null);
	}

	private void refreshPredicateList(
			ECLiPSeInteractionInterface eclipseInteraction, CompoundTerm result) {
		if (activeInteraction == eclipseInteraction) {
			ArrayList predicates = new ArrayList(
					(Collection) ((CompoundTerm) result.arg(2)).arg(4));
			predicateList.removeAll();
			for (int cnt = 0; cnt < predicates.size(); cnt++) {
				predicateList.add((String) predicates.get(cnt));
			}
		}
	}

	private Integer integerFromBoolean(boolean value) {
		if (value) {
			return new Integer(1);
		}

		return new Integer(0);
	}

	private void refreshModuleList(
			ECLiPSeInteractionInterface eclipseInteraction, CompoundTerm result) {
		if (activeInteraction == eclipseInteraction) {
			ArrayList modules = new ArrayList(
					(Collection) ((CompoundTerm) result.arg(2)).arg(1));
			moduleCombo.removeAll();
			for (int cnt = 0; cnt < modules.size(); cnt++) {
				moduleCombo.add(((Atom) modules.get(cnt)).functor());
			}
		}
	}

	public void setFocus() {
	}

	public void configureEngine(
			final ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			requestViewRefresh();
		}
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		activeInteraction = eclipseInteraction;
		requestViewRefresh();
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		deconfigureEngine(eclipseInteraction);
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			activeInteraction = null;
			enableWidgets(false);
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	private ECLiPSeInteractionInterface activeInteraction = null;

	private static String longestLabel = PreferenceConstants.startTracing + ":"; //$NON-NLS-1$ //$NON-NLS-2$

	private static String longestOption = PreferenceConstants.external; //$NON-NLS-1$

	private Group moduleGroup = null;

	private Group predGroup = null;

	private Group propertiesGroup = null;

	private Combo predicateCombo = null;

	private Combo moduleCombo = null;

	private Button filterCheckBox = null;

	private List predicateList = null;

	private Label predicateLabel = null;

	private Label predLabel = null;

	private Label moduleLabel = null;

	private Composite radioEntriesComposite = null;

	private ScrolledComposite viewScrolledComposite = null;

	private Composite viewComposite = null;

	private Menu popupMenu = null;

	private MenuItem refreshViewItem = null;

	private ArrayList buttons = null;

	private ArrayList buttonsEnablementState = null;

	private ArrayList labels = null;

	private Hashtable labelButtonHash = null;
}
