/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.ViewPart;
import org.eclipseclp.saros.development.editors.ECLiPSeExecutionEnvironment;
import org.eclipseclp.saros.runtime.ECLiPSeInteractionInterface;

public class TraceLog extends ViewPart implements SarosViewNotification {

	public void createPartControl(Composite parent) {
		traceLogText = new StyledText(parent, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL | SWT.READ_ONLY);
		IWorkbenchPage page = getSite().getPage();
		if (page == null
				|| !(page.getActiveEditor() instanceof ECLiPSeExecutionEnvironment)) {
			traceLogText.setEnabled(false);
		} else {
			editorActivated((ECLiPSeInteractionInterface) page
					.getActiveEditor());
		}
	}

	public void setFocus() {
		traceLogText.setFocus();
	}

	protected void clearText() {
		traceLogText.replaceTextRange(0, traceLogText.getCharCount(), "");
	}

	public void updateTraceLogData(
			ECLiPSeInteractionInterface eclipseInteraction, String traceText) {
		if (eclipseInteraction == activeInteraction) {
			if (traceText == null) {
				clearText();
			} else {
				traceLogText.append(" " + traceText
						+ traceLogText.getLineDelimiter());
			}
		}
	}

	public void configureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	public void editorActivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (eclipseInteraction == activeInteraction
				&& activeInteraction != null) {
			return;
		}
		activeInteraction = eclipseInteraction;
		traceLogText.setText(eclipseInteraction.getTraceLogData());
		traceLogText.setEnabled(true);
	}

	public void editorDeactivated(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			traceLogText.setEnabled(false);
		}
	}

	public void deconfigureEngine(ECLiPSeInteractionInterface eclipseInteraction) {
		if (activeInteraction == eclipseInteraction) {
			traceLogText.setEnabled(false);
			clearText();
		}
	}

	public void viewClosed(ECLiPSeInteractionInterface eclipseInteraction) {
	}

	StyledText traceLogText = null;

	private ECLiPSeInteractionInterface activeInteraction = null;
}
