/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.wizards;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipseclp.saros.core.preferences.EngineInstallationSelector;
import org.eclipseclp.saros.core.preferences.LaunchQueryComposite;
import org.eclipseclp.saros.core.preferences.CompilerOutput;

public class NewProjectWizardEngineComposite extends Composite implements
		SelectionListener {

	public NewProjectWizardEngineComposite(Composite parent, int style) {
		super(parent, style);
		initialise();
	}

	private void initialise() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		this.setLayout(new GridLayout());
		this.setLayoutData(gridData);
		createEngineGroup();
		createCompilerOutputGroup();
		createLaunchQueryGroup();
		setWidgetValues();
		this.setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createEngineGroup() {
		engineInstallation = new EngineInstallationSelector(this,
				SWT.NONE, true);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		engineInstallation.setLayoutData(gridData);
	}

	private void createCompilerOutputGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		compilerOutputGroup = new CompilerOutput(this, SWT.NONE, true);
		compilerOutputGroup.setLayoutData(gridData);
	}

	private void createLaunchQueryGroup() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		launchQueryGroup = new LaunchQueryComposite(this, SWT.NONE, false);
		launchQueryGroup.setLayoutData(gridData);
	}

	private void setWidgetValues() {
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
	}

	public void commit() {
		engineInstallation.commit();
		isProjectSpecificInstallation = engineInstallation
				.isProjectSpecificInstallation();
		compilerOutputGroup.commit();
		isProjectSpecificCompilationTarget = compilerOutputGroup
				.isProjectSpecificCompilationTarget();
		launchQueryGroup.commit();
	}

	public boolean[] getOutputCompilationTargets() {
		return compilerOutputGroup.getOutputCompilationTargets();
	}

	public boolean hasProjectLaunchQuery() {
		return launchQueryGroup.hasLaunchQuery();
	}

	public boolean isProjectSpecificInstallation() {
		return isProjectSpecificInstallation;
	}

	public boolean isProjectSpecificCompilationTarget() {
		return isProjectSpecificCompilationTarget;
	}

	public String getInstallationName() {
		return engineInstallation.getInstallationName();
	}

	public String getLaunchFileName() {
		return launchQueryGroup.getFileName();
	}

	public String getLaunchModuleName() {
		return launchQueryGroup.getModuleName();
	}

	public String getLaunchQuery() {
		return launchQueryGroup.getQuery();
	}

	public String getLaunchRunMethod() {
		return launchQueryGroup.getRunMethod();
	}

	private EngineInstallationSelector engineInstallation = null;

	private CompilerOutput compilerOutputGroup = null;

	private LaunchQueryComposite launchQueryGroup = null;

	private boolean isProjectSpecificInstallation = false;

	private boolean isProjectSpecificCompilationTarget = false;
}
