/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.wizards;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.ui.dialogs.PreferencesUtil;
import org.eclipseclp.saros.core.preferences.PreferenceManager;
import org.eclipseclp.saros.core.preferences.PreferenceMessages;
import org.eclipseclp.saros.core.preferences.PreferenceConstants;

public class NewProjectWizardProjectComposite extends Composite implements
		SelectionListener {

	public NewProjectWizardProjectComposite(Composite parent, int style) {
		super(parent, style);
		parentComposite = parent;
		initialise();
	}

	private void initialise() {
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		this.setLayout(new GridLayout());
		this.setLayoutData(gridData);
		createProjectNameComposite();
		createProjectContentsGroup();
		createProjectLayoutGroup();
		setWidgetValues();
		this.setSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void createProjectNameComposite() {
		Composite composite = new Composite(this, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		layout.numColumns = 2;
		composite.setLayoutData(gridData);
		projectNameLabel = new Label(composite, SWT.NONE);
		projectNameLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectName")
				+ ":");
		projectNameText = new Text(composite, SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectNameText.setLayoutData(gridData);
	}

	private void createProjectContentsGroup() {
		Composite projectContentsComposite = new Composite(this, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		projectContentsComposite.setLayout(new GridLayout());
		projectContentsComposite.setLayoutData(gridData);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		// layout.verticalSpacing = 8;
		layout.numColumns = 3;
		projectContentsGroup = new Group(projectContentsComposite, SWT.NONE);
		projectContentsGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectContents")); //$NON-NLS-1$
		projectContentsGroup.setLayout(layout);
		projectContentsGroup.setLayoutData(gridData);
		useWorkspaceRadio = new Button(projectContentsGroup, SWT.RADIO);
		useWorkspaceRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CreateWorkspaceProject"));
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 3;
		useWorkspaceRadio.setLayoutData(gridData);
		useWorkspaceRadio.addSelectionListener(this);
		useExistingSourcesRadio = new Button(projectContentsGroup, SWT.RADIO);
		useExistingSourcesRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.CreateExistingSourceProject"));
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 3;
		useExistingSourcesRadio.setLayoutData(gridData);
		useExistingSourcesRadio.addSelectionListener(this);
		existingSourcesLabel = new Label(projectContentsGroup, SWT.NONE);
		gridData = new GridData();
		gridData.horizontalIndent = 22;
		existingSourcesLabel.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Directory")
				+ ":");
		existingSourcesLabel.setLayoutData(gridData);
		existingSourcesText = new Text(projectContentsGroup, SWT.BORDER);
		String defaultLocation = ResourcesPlugin.getWorkspace().getRoot()
				.getLocation().toOSString();
		existingSourcesText.setText(defaultLocation);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		existingSourcesText.setLayoutData(gridData);
		browseExistingSourcesButton = new Button(projectContentsGroup, SWT.PUSH);
		browseExistingSourcesButton.setText(PreferenceMessages
				.getString("ECLiPSeStrings.Browse"));
		browseExistingSourcesButton.addSelectionListener(this);
	}

	private void createProjectLayoutGroup() {
		Composite projectLayoutComposite = new Composite(this, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		GridLayout layout = new GridLayout();
		projectLayoutComposite.setLayout(layout);
		projectLayoutComposite.setLayoutData(gridData);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.grabExcessHorizontalSpace = true;
		layout = new GridLayout();
		// layout.verticalSpacing = 8;
		layout.numColumns = 3;
		projectLayoutGroup = new Group(projectLayoutComposite, SWT.NONE);
		projectLayoutGroup.setText(PreferenceMessages
				.getString("ECLiPSeStrings.ProjectLayout")); //$NON-NLS-1$
		projectLayoutGroup.setLayout(layout);
		projectLayoutGroup.setLayoutData(gridData);
		projectRootRadio = new Button(projectLayoutGroup, SWT.RADIO);
		projectRootRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.UseRootProjectFolder"));
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 3;
		projectRootRadio.setLayoutData(gridData);
		outputFoldersRadio = new Button(projectLayoutGroup, SWT.RADIO);
		outputFoldersRadio.setText(PreferenceMessages
				.getString("ECLiPSeStrings.UseOutputFolders"));
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData.horizontalSpan = 2;
		outputFoldersRadio.setLayoutData(gridData);
		layoutLink = new Link(projectLayoutGroup, SWT.NONE);
		layoutLink.setText("<A>"
				+ PreferenceMessages
						.getString("ECLiPSeStrings.ConfigureDefault")
				+ "...</A>");
		layoutLink.addSelectionListener(this);
	}

	private void setWidgetValues() {
		useWorkspaceRadio.setSelection(true);
		useExistingSourcesRadio.setSelection(false);
		existingSourcesLabel.setEnabled(false);
		existingSourcesText.setEnabled(false);
		browseExistingSourcesButton.setEnabled(false);

		if (PreferenceManager.projectFolderOutput()) {
			projectRootRadio.setSelection(true);
			outputFoldersRadio.setSelection(false);
		} else {
			projectRootRadio.setSelection(false);
			outputFoldersRadio.setSelection(true);
		}
	}

	public void widgetDefaultSelected(SelectionEvent event) {
	}

	public void widgetSelected(SelectionEvent event) {
		if (event.getSource() == useWorkspaceRadio) {
			useWorkspaceRadio.setSelection(true);
			useExistingSourcesRadio.setSelection(false);
			existingSourcesLabel.setEnabled(false);
			existingSourcesText.setEnabled(false);
			String defaultLocation = ResourcesPlugin.getWorkspace().getRoot()
					.getLocation().toOSString();
			existingSourcesText.setText(defaultLocation);
			browseExistingSourcesButton.setEnabled(false);
			projectRootRadio.setEnabled(true);
			outputFoldersRadio.setEnabled(true);
			layoutLink.setEnabled(true);
		} else if (event.getSource() == useExistingSourcesRadio) {
			useWorkspaceRadio.setSelection(false);
			useExistingSourcesRadio.setSelection(true);
			existingSourcesLabel.setEnabled(true);
			existingSourcesText.setEnabled(true);
			existingSourcesText.setText("");
			browseExistingSourcesButton.setEnabled(true);
			projectRootRadio.setEnabled(false);
			outputFoldersRadio.setEnabled(false);
			layoutLink.setEnabled(false);
		} else if (event.getSource() == layoutLink) {
			PreferencesUtil
					.createPreferenceDialogOn(
							getShell(),
							PreferenceConstants.BuildPathPreferencePageId,
							new String[] { PreferenceConstants.BuildPathPreferencePageId },
							null).open();
			setWidgetValues();
		} else if (event.getSource() == browseExistingSourcesButton) {
			DirectoryDialog dialog = new DirectoryDialog(parentComposite
					.getShell(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
			dialog.setText(PreferenceMessages
					.getString("ECLiPSeStrings.BrowseFolder"));
			dialog.setMessage(PreferenceMessages
					.getString("ECLiPSeStrings.SelectExistingSourcesDir")
					+ ":");
			dialog.setFilterPath("");
			String projectPath = dialog.open();
			if (projectPath != null) {
				existingSourcesText.setText(projectPath);
			}
		}
	}

	public void addProjectNameModifyListener(ModifyListener listener) {
		projectNameText.addModifyListener(listener);
	}

	public void addExistingSourcesModifyListener(ModifyListener listener,
			SelectionAdapter adapter) {
		existingSourcesText.addModifyListener(listener);
		useWorkspaceRadio.addSelectionListener(adapter);
		useExistingSourcesRadio.addSelectionListener(adapter);
		useWorkspaceRadio.addSelectionListener(adapter);
	}

	public void commit() {
		projectName = projectNameText.getText();
		projectPath = existingSourcesText.getText();
		isProjectFolderOutput = projectRootRadio.getSelection();
	}

	public boolean useExistingSourcesWizardPageValidator() {
		return useExistingSourcesRadio.getSelection();
	}

	public String getExistingSourcesText() {
		return existingSourcesText.getText();
	}

	public String getProjectNameText() {
		return projectNameText.getText();
	}

	public String getProjectName() {
		return projectName;
	}

	public String getProjectPath() {
		return projectPath;
	}

	public String[] getOutputFolders() {
		if (isProjectFolderOutput) {
			return new String[0];
		}
		return PreferenceManager.getOutputFolders();
	}

	public boolean isProjectFolderOutput() {
		return isProjectFolderOutput;
	}

	private Label projectNameLabel = null;

	private Text projectNameText = null;

	private Group projectContentsGroup = null;

	private Button useWorkspaceRadio = null;

	private Button useExistingSourcesRadio = null;

	private Label existingSourcesLabel = null;

	private Button browseExistingSourcesButton = null;

	private Text existingSourcesText = null;

	private Group projectLayoutGroup = null;

	private Button projectRootRadio = null;

	private Button outputFoldersRadio = null;

	private Link layoutLink = null;

	private Composite parentComposite = null;

	private String projectName = "";

	private String projectPath = "";

	private boolean isProjectFolderOutput = true;
}
