/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.development.wizards;

import java.io.File;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;

public class NewProjectWizardProjectPage extends WizardPage {

	public NewProjectWizardProjectPage(ISelection selection) {
		super("newSarosProjectWizardProjectPage");
		setTitle("New Saros Project");
		setDescription("Create a Saros project in the workspace or an external location");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		projectWizardComposite = new NewProjectWizardProjectComposite(parent, SWT.NONE);
		ModifyListener ml = new ModifyListener() {
			public void modifyText(ModifyEvent event) {
				dialogChanged();
			}
		};
		projectWizardComposite.addProjectNameModifyListener(ml);
		projectWizardComposite.addExistingSourcesModifyListener(ml,
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent event) {
						dialogChanged();
					}
				});
		dialogChanged();
		setControl(projectWizardComposite);
	}

	/**
	 * Ensures that both text fields are set.
	 */

	protected void dialogChanged() {
		String projectName = projectWizardComposite.getProjectNameText();
		if (projectName.length() == 0) {
			updateStatus("Project name must be specified");
			setPageComplete(false);
			return;
		}

		IResource container = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(new Path(projectName));

		if (container != null
				&& (container.getType() & (IResource.PROJECT)) != 0) {
			updateStatus("A project '" + projectName + "' already exists");
			setPageComplete(false);
			return;
		}

		if (projectWizardComposite.getExistingSourcesText().equals("")) {
			updateStatus("An existing source directory must exist");
			setPageComplete(false);
			return;
		}

		updateStatus(null);
		setPageComplete(true);
	}

	public boolean canFlipToNextPage() {
		return super.canFlipToNextPage()
				&& isPageComplete()
				&& (!projectWizardComposite
						.useExistingSourcesWizardPageValidator() || (projectWizardComposite
						.useExistingSourcesWizardPageValidator() && !(new File(
						projectWizardComposite.getExistingSourcesText()))
						.exists()));
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	public String getProjectNameText() {
		return projectWizardComposite.getProjectNameText();
	}

	public NewProjectWizardProjectComposite getWizardComposite() {
		return projectWizardComposite;
	}

	private NewProjectWizardProjectComposite projectWizardComposite = null;
}
