/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.runtime;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IMarker;

/**
 * @author amc
 * 
 */
public class ECLiPSeCompilationError {

	public ECLiPSeCompilationError(String message, int severity) {
		this.severity = severity;
		if (severity == IMarker.SEVERITY_ERROR) {
			parseCompilationError(message);
		} else {
			parseCompilationWarning(message);
		}
	}

	public ECLiPSeCompilationError(String message, int severity,
			String sourceFolder) {
		this(message, severity);
		if (file != null && !sourceFolder.equals("")) {
			// '/' is eclipse relative path delimeter, not file system
			file = sourceFolder + "/" + file;
		}
	}

	private void parseCompilationError(String message) {
		int group = 0;
		String regex = "\\Afile\\s+(.+?),\\s+line\\s+(\\d+):(.*?):(.*?)\n\\|\\s(.*?)\n\\|\\s(.*?)\n$";
		// String regex =
		// "\\Afile\\s+(.+?),\\s+line\\s+(\\d+):(.*?):(.*?)\n(.*?)\n$";
		String[] errorDetail = null;
		Matcher matcher = null;
		try {
			Pattern pattern = Pattern.compile(regex, Pattern.DOTALL);
			matcher = pattern.matcher(message);
			matcher.find();
			errorDetail = new String[matcher.groupCount()];
			// matcher.group(0) is entire match!
			for (group = 1; group <= errorDetail.length; group++) {
				errorDetail[group - 1] = matcher.group(group);
			}
		} catch (Exception e) {
			detail = message;
			return;
		}
		file = errorDetail[0].trim();
		lineNumber = new Integer(errorDetail[1].trim()).intValue();
		type = errorDetail[2].trim();
		reason = errorDetail[3].trim();
		if (errorDetail[4] != null & errorDetail[5] != null) {
			startChar = errorDetail[5].indexOf('^');
			if (startChar != -1) {
				endChar = errorDetail[4].length() - 1;
			}
			detail = "| " + errorDetail[4] + "\n| " + errorDetail[5];
		} else {
			detail = errorDetail[4].trim();
		}
	}

	private void parseCompilationWarning(String message) {
		int index = message.toLowerCase().indexOf("warning:");
		if (index != -1) {
			detail = message.substring(index + 8);
		} else {
			// Not the warning format we expect
			detail = message;
		}
	}

	public void printCompilationError() {
		System.err.println("Compilation Error with severity: " + severity);
		System.err.println("\tFilename: " + file);
		System.err.println("\tLine number: " + lineNumber);
		System.err.println("\tType: " + type);
		System.err.println("\tReason: " + reason);
		System.err.println("\tDetail: " + detail);
	}

	public String getDetail() {
		return detail;
	}

	public void setDetail(String detail) {
		this.detail = detail;
	}

	public String getFile() {
		return file;
	}

	public void setFile(String file) {
		this.file = file;
	}

	public String getReason() {
		return reason;
	}

	public void setReason(String reason) {
		this.reason = reason;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public int getLineNumber() {
		return lineNumber;
	}

	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}

	public int getSeverity() {
		return severity;
	}

	public void setSeverity(int severity) {
		this.severity = severity;
	}

	public int getEndChar() {
		return endChar;
	}

	public void setEndChar(int endChar) {
		this.endChar = endChar;
	}

	public int getStartChar() {
		return startChar;
	}

	public void setStartChar(int startChar) {
		this.startChar = startChar;
	}

	private int lineNumber = -1;

	private int startChar = -1;

	private int endChar = -1;

	private String file = null;

	private String type = null;

	private String reason = null;

	private String detail = null;

	private int severity = -1;
}
