/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.runtime;

import java.util.ArrayList;

import org.eclipseclp.saros.core.preferences.EnginePreferences;

import com.parctechnologies.eclipse.QueueListener;

public interface ECLiPSeInteractionInterface extends
		ToECLiPSeInteractionInterface {
	public abstract QueueListener getOutputStreamListener();

	public abstract QueueListener getErrorStreamListener();

	public abstract QueueListener getWarningStreamListener();

	public abstract QueueListener getAnswerStreamListener();

	public abstract QueueListener getStatisticsStreamListener();

	public abstract String requestInput();

	public abstract void requestToplevelStateTransition(int state);

	public abstract void reflectStoppedStatus();

	public abstract void reflectMoreStatus();

	public abstract void setActiveModules(String[] moduleList,
			String selectedModule);

	public abstract void sendEngineReconfigurationNotification();

	public abstract void reflectTracerStatus(boolean enabled);

	public abstract int getToplevelState();

	public abstract ArrayList getInspectorData();

	public abstract void inspectCurrentGoal();

	public abstract void inspectInvokedGoal(int invocation);

	public abstract void updateCallStackData(ECLiPSeTraceLine traceLine);

	public abstract void traceSource(ECLiPSeTraceLine traceLine);

	public abstract ArrayList getCallStackData();

	public abstract void forceGoalFailure(ECLiPSeTraceLine traceLine);

	public abstract void analyseGoalFailure(ECLiPSeTraceLine traceLine);

	public abstract void jumpInvocationNumber(ECLiPSeTraceLine traceLine);

	public abstract void jumpInvocationDepth(ECLiPSeTraceLine traceLine);

	public abstract void requestCallStackRefresh();

	public boolean isAutoRefreshingTracer();

	public void setAutoRefreshingTracer(boolean autoRefresh);

	public abstract void updateTraceLogData(String traceText);

	public abstract String getTraceLogData();

	public abstract void updateStatisticsData(String data);

	public abstract void updateCompileScratchpadData(String data);

	public abstract String getCompileScratchpadData();

	public abstract void updateDelayedGoalsData(ArrayList delayedGoals);

	public abstract ArrayList getDelayedGoalsData();

	public boolean isAutoRefreshingDelayedGoals();

	public void setAutoRefreshingDelayedGoals(boolean autoRefresh);

	public abstract void setStatisticsRefreshRate(Float refreshRate);

	public abstract Float getStatisticsRefreshRate();

	public abstract boolean statisticsReportingEnabled();

	public abstract void setStatisticsReportingEnabled(boolean enabled);

	public abstract void setStatisticsDisplay(Object display);

	public abstract void shutdown();

	public abstract EnginePreferences getInstallation();

	public abstract boolean getMultipleExecutionEnvironmentInstanceSupport();

	public abstract void activate();

	public abstract void deactivate();

	public static final String ECLIPSE_TOPLEVEL_INIT = "lib(toplevel), toplevel_init(gui)";

	public static final String ECLIPSE_TOPLEVEL_QUERY = "toplevel";

	public static final int ECLIPSE_STATE_RUNNING = 0;

	public static final int ECLIPSE_STATE_YES = 1;

	public static final int ECLIPSE_STATE_MORE = 2;

	public static final int ECLIPSE_STATE_IDLE = 3;

	public static final int ECLIPSE_REQUEST_CALL = 0;

	public static final int ECLIPSE_REQUEST_MORE = 1;

	public static final int ECLIPSE_REQUEST_END = 2;

	public static final int ECLIPSE_REQUEST_EXIT = 3;

	public static final String ECLIPSE_YES_STRING = "Yes";

	public static final String ECLIPSE_NO_STRING = "No";

	public static final String ECLIPSE_ABORT_STRING = "Abort";

	public static final String ECLIPSE_MORE_STRING = "More";

	public static final String ECLIPSE_IDLE_STRING = "Idle";

	public static final String ECLIPSE_TOP_LEVEL_IN_Q = "toplevel_in";

	public static final String ECLIPSE_TOP_LEVEL_OUT_Q = "toplevel_out";

	public static final String ECLIPSE_ANSWER_OUT_Q = "answer_output";

	public static final String ECLIPSE_WARNING_OUT_Q = "warning_output";

	public static final String ECLIPSE_GUI_INTERRUPT_MULTITASK_PHASE = "gui_interrupt";

	public static final String ECLIPSE_INT_REQ_Q = "gui_interrupt_request";

	public static final String ECLIPSE_PAUSE_REQ_Q = "gui_pause_request";

	public static final String ECLIPSE_DEBUG_TRACELINE_Q = "debug_traceline";

	public static final String ECLIPSE_DEBUG_OUTPUT_Q = "debug_output";

	public static final String ECLIPSE_TRACER_MULTITASK_PHASE = "tracer";

	public static final String ECLIPSE_STATISTICS_Q = "statistics_text_summary_queue";

	public static final String ECLIPSE_DELAYED_GOAL_Q = "gui_dg_info";

	public int state = ECLIPSE_STATE_IDLE;
}
