/*******************************************************************************
 * Copyright (c) 2006 Cisco Technology, Inc and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author: Andrew Cheadle, CrossCore Optimization Limited
 *
 * Contributors:
 *     Cisco Technology, Inc - Initial implementation
 *******************************************************************************/

package org.eclipseclp.saros.runtime;

import com.parctechnologies.eclipse.QueueListener;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.custom.StyledText;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import org.eclipseclp.saros.runtime.RunnableTextAreaUpdater;

public class ECLiPSeStdStreamDataListener implements QueueListener {

	public ECLiPSeStdStreamDataListener(StyledText displayTextArea, int colour,
			boolean clearOnUpdate, boolean asyncUpdate) {
		this.displayTextArea = displayTextArea;
		this.colour = colour;
		this.clearOnUpdate = clearOnUpdate;
		this.asyncUpdate = asyncUpdate;
	}

	public void dataAvailable(Object source) {
		// System.out.println("Data available on: " + source.toString());
		if (bufferedInputStream == null) {
			bufferedInputStream = new BufferedInputStream((InputStream) source);
		}

		try {
			int numBytes = bufferedInputStream.available();
			byte[] bytes = new byte[numBytes];
			if (bufferedInputStream.read(bytes, 0, numBytes) != -1) {
				RunnableTextAreaUpdater rtau = new RunnableTextAreaUpdater(
						new String(bytes), displayTextArea, colour,
						clearOnUpdate);
				if (asyncUpdate) {
					Display.getDefault().asyncExec(rtau);
				} else {
					Display.getDefault().syncExec(rtau);
				}
				charCount = rtau.getCharCount();
			}
		} catch (IOException ioe) {
			System.err.println("I/O exception caught whilst reading "
					+ "data from Saros ECLiPSe engine: " + ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	public void dataRequest(Object source) {
	}

	public int getCharCount() {
		return charCount;
	}

	public void setTextArea(StyledText textArea) {
		this.displayTextArea = textArea;
	}

	private BufferedInputStream bufferedInputStream = null;

	private StyledText displayTextArea = null;

	private int colour;

	private int charCount = 0;

	private boolean clearOnUpdate = false;

	private boolean asyncUpdate = false;
}
